"""
    :codeauthor: Nicole Thomas <nicole@saltstack.com>
"""

import os
import random

import pytest
import salt.utils.files
from salt.exceptions import CommandExecutionError
from tests.support.case import ModuleCase
from tests.support.helpers import runs_on

# Module Variables
ASSIGN_CMD = "net.inet.icmp.icmplim"
CONFIG = "/etc/sysctl.conf"


@pytest.mark.destructive_test
@runs_on(kernel="Darwin")
@pytest.mark.skip_if_not_root
class DarwinSysctlModuleTest(ModuleCase):
    """
    Integration tests for the darwin_sysctl module
    """

    def setUp(self):
        """
        Sets up the test requirements
        """
        super().setUp()
        # Data needed for cleanup
        self.has_conf = False
        self.val = self.run_function("sysctl.get", [ASSIGN_CMD])

        # If sysctl file is present, make a copy
        # Remove original file so we can replace it with test files
        if os.path.isfile(CONFIG):
            self.has_conf = True
            try:
                self.conf = self.__copy_sysctl()
            except CommandExecutionError:
                msg = "Could not copy file: {0}"
                raise CommandExecutionError(msg.format(CONFIG))
            os.remove(CONFIG)

    @pytest.mark.slow_test
    def test_assign(self):
        """
        Tests assigning a single sysctl parameter
        """
        try:
            rand = random.randint(0, 500)
            while rand == self.val:
                rand = random.randint(0, 500)
            self.run_function("sysctl.assign", [ASSIGN_CMD, rand])
            info = int(self.run_function("sysctl.get", [ASSIGN_CMD]))
            try:
                self.assertEqual(rand, info)
            except AssertionError:
                self.run_function("sysctl.assign", [ASSIGN_CMD, self.val])
                raise
        except CommandExecutionError:
            self.run_function("sysctl.assign", [ASSIGN_CMD, self.val])
            raise

    @pytest.mark.slow_test
    def test_persist_new_file(self):
        """
        Tests assigning a sysctl value to a system without a sysctl.conf file
        """
        # Always start with a clean/known sysctl.conf state
        if os.path.isfile(CONFIG):
            os.remove(CONFIG)
        try:
            self.run_function("sysctl.persist", [ASSIGN_CMD, 10])
            line = "{}={}".format(ASSIGN_CMD, 10)
            found = self.__check_string(CONFIG, line)
            self.assertTrue(found)
        except CommandExecutionError:
            os.remove(CONFIG)
            raise

    @pytest.mark.slow_test
    def test_persist_already_set(self):
        """
        Tests assigning a sysctl value that is already set in sysctl.conf file
        """
        # Always start with a clean/known sysctl.conf state
        if os.path.isfile(CONFIG):
            os.remove(CONFIG)
        try:
            self.run_function("sysctl.persist", [ASSIGN_CMD, 50])
            ret = self.run_function("sysctl.persist", [ASSIGN_CMD, 50])
            self.assertEqual(ret, "Already set")
        except CommandExecutionError:
            os.remove(CONFIG)
            raise

    @pytest.mark.slow_test
    def test_persist_apply_change(self):
        """
        Tests assigning a sysctl value and applying the change to system
        """
        # Always start with a clean/known sysctl.conf state
        if os.path.isfile(CONFIG):
            os.remove(CONFIG)
        try:
            rand = random.randint(0, 500)
            while rand == self.val:
                rand = random.randint(0, 500)
            self.run_function("sysctl.persist", [ASSIGN_CMD, rand], apply_change=True)
            info = int(self.run_function("sysctl.get", [ASSIGN_CMD]))
            self.assertEqual(info, rand)
        except CommandExecutionError:
            os.remove(CONFIG)
            raise

    def __copy_sysctl(self):
        """
        Copies an existing sysconf file and returns temp file path. Copied
        file will be restored in tearDown
        """
        # Create new temporary file path and open needed files
        temp_path = salt.utils.files.mkstemp()
        with salt.utils.files.fopen(CONFIG, "r") as org_conf:
            with salt.utils.files.fopen(temp_path, "w") as temp_sysconf:
                # write sysctl lines to temp file
                for line in org_conf:
                    temp_sysconf.write(line)
        return temp_path

    def __restore_sysctl(self):
        """
        Restores the original sysctl.conf file from temporary copy
        """
        # If sysctl testing file exists, delete it
        if os.path.isfile(CONFIG):
            os.remove(CONFIG)

        # write temp lines to sysctl file to restore
        with salt.utils.files.fopen(self.conf, "r") as temp_sysctl:
            with salt.utils.files.fopen(CONFIG, "w") as sysctl:
                for line in temp_sysctl:
                    sysctl.write(line)

        # delete temporary file
        os.remove(self.conf)

    def __check_string(self, conf_file, to_find):
        """
        Returns True if given line is present in file
        """
        with salt.utils.files.fopen(conf_file, "r") as f_in:
            for line in f_in:
                if to_find in salt.utils.stringutils.to_unicode(line):
                    return True
            return False

    def tearDown(self):
        """
        Clean up after tests
        """
        ret = self.run_function("sysctl.get", [ASSIGN_CMD])
        if ret != self.val:
            self.run_function("sysctl.assign", [ASSIGN_CMD, self.val])

        if self.has_conf is True:
            # restore original sysctl file
            self.__restore_sysctl()

        if self.has_conf is False and os.path.isfile(CONFIG):
            # remove sysctl.conf created by tests
            os.remove(CONFIG)



import pytest
import salt.utils.platform
from tests.support.case import ModuleCase
from tests.support.unit import skipIf


@skipIf(salt.utils.platform.is_windows(), "minion is windows")
@skipIf(salt.utils.platform.is_darwin(), "locale method is not supported on mac")
@skipIf(
    salt.utils.platform.is_freebsd(),
    "locale method is supported only within login classes or environment variables",
)
@pytest.mark.requires_salt_modules("locale")
@pytest.mark.windows_whitelisted
class LocaleModuleTest(ModuleCase):
    def _find_new_locale(self, current_locale):
        test_locales = ["en_US.UTF-8", "de_DE.UTF-8", "fr_FR.UTF-8", "en_AU.UTF-8"]
        for locale in test_locales:
            if locale != current_locale and self.run_function("locale.avail", [locale]):
                return locale

        self.skipTest(
            "The test locals: {} do not exist on the host. Skipping test.".format(
                ",".join(test_locales)
            )
        )

    def test_get_locale(self):
        locale = self.run_function("locale.get_locale")
        self.assertNotIn("Unsupported platform!", locale)

    @pytest.mark.destructive_test
    @pytest.mark.slow_test
    def test_gen_locale(self):
        # Make sure charmaps are available on test system before attempting
        # call gen_locale. We log this error to the user in the function, but
        # we don't want to fail this test if this is missing on the test system.
        char_maps = self.run_function("cmd.run_all", ["locale -m"])
        if char_maps["stdout"] == "":
            self.skipTest("locale charmaps not available. Skipping test.")

        if char_maps["retcode"] and char_maps["stderr"]:
            self.skipTest(
                "{}. Cannot generate locale. Skipping test.".format(char_maps["stderr"])
            )

        locale = self.run_function("locale.get_locale")
        new_locale = self._find_new_locale(locale)
        ret = self.run_function("locale.gen_locale", [new_locale])
        self.assertTrue(ret)

    @pytest.mark.destructive_test
    @pytest.mark.slow_test
    def test_set_locale(self):
        original_locale = self.run_function("locale.get_locale")
        locale_to_set = self._find_new_locale(original_locale)
        self.run_function("locale.gen_locale", [locale_to_set])
        ret = self.run_function("locale.set_locale", [locale_to_set])
        new_locale = self.run_function("locale.get_locale")
        self.assertTrue(ret)
        self.assertEqual(locale_to_set, new_locale)
        self.run_function("locale.set_locale", [original_locale])



"""
Display clean output of an overstate stage
==========================================

This outputter is used to display :ref:`Orchestrate Runner
<orchestrate-runner>` stages, and should not be called directly.
"""


import salt.utils.color

# [{'group2': {'match': ['fedora17-2', 'fedora17-3'],
#              'require': ['group1'],
#              'sls': ['nginx', 'edit']}
#              }
#              ]


def output(data, **kwargs):  # pylint: disable=unused-argument
    """
    Format the data for printing stage information from the overstate system
    """
    colors = salt.utils.color.get_colors(
        __opts__.get("color"), __opts__.get("color_theme")
    )
    ostr = ""
    for comp in data:
        for name, stage in comp.items():
            ostr += "{}{}: {}\n".format(colors["LIGHT_BLUE"], name, colors["ENDC"])
            for key in sorted(stage):
                ostr += "    {}{}: {}{}\n".format(
                    colors["LIGHT_BLUE"], key, stage[key], colors["ENDC"]
                )
    return ostr



from salt.cloud.clouds import vultrpy as vultr
from tests.support.helpers import TstSuiteLoggingHandler
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class VultrTestCase(TestCase, LoaderModuleMockMixin):
    """
    Unit TestCase for salt.cloud.clouds.vultr module.
    """

    def setup_loader_modules(self):
        provider_info = {
            "vultr01": {"vultr": {"api_key": "super_secret_key", "driver": "vultr"}}
        }

        opts = {"providers": provider_info}

        return {
            vultr: {
                "__utils__": {
                    "cloud.fire_event": MagicMock(),
                    "cloud.filter_event": MagicMock(),
                    "cloud.wait_for_fun": MagicMock(),
                    "cloud.bootstrap": MagicMock(),
                },
                "__opts__": {
                    "providers": provider_info,
                    "sock_dir": "/tmp/sock_dir",
                    "transport": "tcp",
                },
                "__active_provider_name__": "my_vultr:vultr",
            }
        }

    def test_show_keypair_no_keyname(self):
        """
        test salt.cloud.clouds.vultr.show_keypair
        when keyname is not in kwargs
        """
        kwargs = {}
        with TstSuiteLoggingHandler() as handler:
            assert not vultr.show_keypair(kwargs)
            assert "ERROR:A keyname is required." in handler.messages

    @patch("salt.cloud.clouds.vultrpy._query")
    def test_show_keypair(self, _query):
        """
        test salt.cloud.clouds.vultr.show_keypair
        when keyname provided
        """
        _query.return_value = {"test": {"SSHKEYID": "keyID"}}
        kwargs = {"keyname": "test"}
        assert vultr.show_keypair(kwargs) == {"SSHKEYID": "keyID"}

    def test_create_firewall_ssh(self):
        """
        Test create when setting firewall_group_id and
        ssh_key_names.
        """
        kwargs = {
            "provider": "vultr",
            "enable_private_network": True,
            "ssh_key_names": "key1,key2,key3",
            "startup_script_id": "test_id",
            "firewall_group_id": "f_id",
            "image": 223,
            "size": 13,
            "location": 1,
            "name": "test-vm",
        }
        patch_scripts = patch(
            "salt.cloud.clouds.vultrpy.avail_scripts",
            MagicMock(return_value=["test_id"]),
        )

        patch_firewall = patch(
            "salt.cloud.clouds.vultrpy.avail_firewall_groups",
            MagicMock(return_value=["f_id"]),
        )

        patch_keys = patch(
            "salt.cloud.clouds.vultrpy.avail_keys",
            MagicMock(return_value=["key3", "key2", "key1"]),
        )

        patch_vultrid = patch(
            "salt.cloud.clouds.vultrpy._lookup_vultrid",
            MagicMock(return_value="test_id"),
        )

        mock_query = MagicMock(return_value={"status": 200})
        patch_query = patch("salt.cloud.clouds.vultrpy._query", mock_query)

        patch_show = patch("salt.cloud.clouds.vultrpy.show_instance", MagicMock())

        with patch_scripts, patch_firewall, patch_keys, patch_vultrid, patch_query, patch_show:
            vultr.create(kwargs)
            query_ret = mock_query.call_args.kwargs["data"]
            self.assertIn("SSHKEYID=key1%2Ckey2%2Ckey3", query_ret)
            self.assertIn("FIREWALLGROUPID=f_id", query_ret)

    def test_create_firewall_doesnotexist(self):
        """
        Test create when setting firewall_group_id to a firewall
        that does not exist
        """
        kwargs = {
            "provider": "vultr",
            "enable_private_network": True,
            "startup_script_id": "test_id",
            "firewall_group_id": "doesnotexist",
            "image": 223,
            "size": 13,
            "location": 1,
            "name": "test-vm",
        }
        patch_scripts = patch(
            "salt.cloud.clouds.vultrpy.avail_scripts",
            MagicMock(return_value=["test_id"]),
        )

        patch_firewall = patch(
            "salt.cloud.clouds.vultrpy.avail_firewall_groups",
            MagicMock(return_value=["f_id"]),
        )

        patch_keys = patch(
            "salt.cloud.clouds.vultrpy.avail_keys",
            MagicMock(return_value=["key3", "key2", "key1"]),
        )

        patch_vultrid = patch(
            "salt.cloud.clouds.vultrpy._lookup_vultrid",
            MagicMock(return_value="test_id"),
        )

        mock_query = MagicMock(return_value={"status": 200})
        patch_query = patch("salt.cloud.clouds.vultrpy._query", mock_query)

        patch_show = patch("salt.cloud.clouds.vultrpy.show_instance", MagicMock())

        with patch_scripts, patch_firewall, patch_keys, patch_vultrid, patch_query, patch_show:
            with TstSuiteLoggingHandler() as handler:
                ret = vultr.create(kwargs)
                self.assertIn(
                    "ERROR:Your Vultr account does not have a firewall group with ID"
                    " doesnotexist",
                    handler.messages,
                )
                self.assertFalse(ret)

    def test_create_ssh_key_ids_doesnotexist(self):
        """
        Test create when setting ssh_key_ids that do not
        exist
        """
        kwargs = {
            "provider": "vultr",
            "enable_private_network": True,
            "startup_script_id": "test_id",
            "ssh_key_names": "doesnotexist",
            "image": 223,
            "size": 13,
            "location": 1,
            "name": "test-vm",
        }
        patch_scripts = patch(
            "salt.cloud.clouds.vultrpy.avail_scripts",
            MagicMock(return_value=["test_id"]),
        )

        patch_firewall = patch(
            "salt.cloud.clouds.vultrpy.avail_firewall_groups",
            MagicMock(return_value=["f_id"]),
        )

        patch_keys = patch(
            "salt.cloud.clouds.vultrpy.avail_keys",
            MagicMock(return_value=["key3", "key2", "key1"]),
        )

        patch_vultrid = patch(
            "salt.cloud.clouds.vultrpy._lookup_vultrid",
            MagicMock(return_value="test_id"),
        )

        mock_query = MagicMock(return_value={"status": 200})
        patch_query = patch("salt.cloud.clouds.vultrpy._query", mock_query)

        patch_show = patch("salt.cloud.clouds.vultrpy.show_instance", MagicMock())

        with patch_scripts, patch_firewall, patch_keys, patch_vultrid, patch_query, patch_show:
            with TstSuiteLoggingHandler() as handler:
                ret = vultr.create(kwargs)
                self.assertIn(
                    "ERROR:Your Vultr account does not have a key with ID doesnotexist",
                    handler.messages,
                )
                self.assertFalse(ret)



"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""

import sys

import salt.states.linux_acl as linux_acl
from salt.exceptions import CommandExecutionError
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase, skipIf


@skipIf(not sys.platform.startswith("linux"), "Test for Linux only")
class LinuxAclTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.linux_acl
    """

    def setup_loader_modules(self):
        return {linux_acl: {}}

    # 'present' function tests: 1

    def test_present(self):
        """
        Test to ensure a Linux ACL is present
        """
        self.maxDiff = None
        name = "/root"
        acl_type = "users"
        acl_name = "damian"
        perms = "rwx"

        mock = MagicMock(
            side_effect=[
                {name: {acl_type: [{acl_name: {"octal": 5}}]}},
                {name: {acl_type: [{acl_name: {"octal": 5}}]}},
                {name: {acl_type: [{acl_name: {"octal": 5}}]}},
                {name: {acl_type: [{}]}},
                {name: {acl_type: [{}]}},
                {name: {acl_type: [{}]}},
                {
                    name: {acl_type: [{acl_name: {"octal": 7}}]},
                    name + "/foo": {acl_type: [{acl_name: {"octal": 5}}]},
                },
                {
                    name: {acl_type: [{acl_name: {"octal": 7}}]},
                    name + "/foo": {acl_type: [{acl_name: {"octal": 7}}]},
                },
                {name: {acl_type: ""}},
                {
                    name: {"defaults": {"users": [{acl_name: {"octal": 7}}]}},
                    name + "/foo": {"defaults": {"users": [{acl_name: {"octal": 7}}]}},
                },
                {
                    name: {"defaults": {"users": [{acl_name: {"octal": 7}}]}},
                    name + "/foo": {"defaults": {"users": [{acl_name: {"octal": 7}}]}},
                },
                {
                    name: {"defaults": {"users": [{acl_name: {"octal": 7}}]}},
                    name + "/foo": {"defaults": {"users": [{acl_name: {"octal": 7}}]}},
                },
            ]
        )
        mock_modfacl = MagicMock(return_value=True)

        with patch.dict(linux_acl.__salt__, {"acl.getfacl": mock}):
            # Update - test=True
            with patch.dict(linux_acl.__opts__, {"test": True}):
                comt = "Updated permissions will be applied for {}: r-x -> {}".format(
                    acl_name, perms
                )
                ret = {
                    "name": name,
                    "comment": comt,
                    "changes": {
                        "new": {
                            "acl_name": acl_name,
                            "acl_type": acl_type,
                            "perms": perms,
                        },
                        "old": {
                            "acl_name": acl_name,
                            "acl_type": acl_type,
                            "perms": "r-x",
                        },
                    },
                    "result": None,
                }

                self.assertDictEqual(
                    linux_acl.present(name, acl_type, acl_name, perms), ret
                )
            # Update - test=False
            with patch.dict(linux_acl.__salt__, {"acl.modfacl": mock_modfacl}):
                with patch.dict(linux_acl.__opts__, {"test": False}):
                    comt = "Updated permissions for {}".format(acl_name)
                    ret = {
                        "name": name,
                        "comment": comt,
                        "changes": {
                            "new": {
                                "acl_name": acl_name,
                                "acl_type": acl_type,
                                "perms": perms,
                            },
                            "old": {
                                "acl_name": acl_name,
                                "acl_type": acl_type,
                                "perms": "r-x",
                            },
                        },
                        "result": True,
                    }
                    self.assertDictEqual(
                        linux_acl.present(name, acl_type, acl_name, perms), ret
                    )
            # Update - modfacl error
            with patch.dict(
                linux_acl.__salt__,
                {
                    "acl.modfacl": MagicMock(
                        side_effect=CommandExecutionError("Custom err")
                    )
                },
            ):
                with patch.dict(linux_acl.__opts__, {"test": False}):
                    comt = "Error updating permissions for {}: Custom err".format(
                        acl_name
                    )
                    ret = {
                        "name": name,
                        "comment": comt,
                        "changes": {},
                        "result": False,
                    }
                    self.assertDictEqual(
                        linux_acl.present(name, acl_type, acl_name, perms), ret
                    )
            # New - test=True
            with patch.dict(linux_acl.__salt__, {"acl.modfacl": mock_modfacl}):
                with patch.dict(linux_acl.__opts__, {"test": True}):
                    comt = "New permissions will be applied for {}: {}".format(
                        acl_name, perms
                    )
                    ret = {
                        "name": name,
                        "comment": comt,
                        "changes": {
                            "new": {
                                "acl_name": acl_name,
                                "acl_type": acl_type,
                                "perms": perms,
                            }
                        },
                        "result": None,
                    }
                    self.assertDictEqual(
                        linux_acl.present(name, acl_type, acl_name, perms), ret
                    )
            # New - test=False
            with patch.dict(linux_acl.__salt__, {"acl.modfacl": mock_modfacl}):
                with patch.dict(linux_acl.__opts__, {"test": False}):
                    comt = "Applied new permissions for {}".format(acl_name)
                    ret = {
                        "name": name,
                        "comment": comt,
                        "changes": {
                            "new": {
                                "acl_name": acl_name,
                                "acl_type": acl_type,
                                "perms": perms,
                            }
                        },
                        "result": True,
                    }
                    self.assertDictEqual(
                        linux_acl.present(name, acl_type, acl_name, perms), ret
                    )
            # New - modfacl error
            with patch.dict(
                linux_acl.__salt__,
                {
                    "acl.modfacl": MagicMock(
                        side_effect=CommandExecutionError("Custom err")
                    )
                },
            ):
                with patch.dict(linux_acl.__opts__, {"test": False}):
                    comt = "Error updating permissions for {}: Custom err".format(
                        acl_name
                    )
                    ret = {
                        "name": name,
                        "comment": comt,
                        "changes": {},
                        "result": False,
                    }
                    self.assertDictEqual(
                        linux_acl.present(name, acl_type, acl_name, perms), ret
                    )

            # New - recurse true
            with patch.dict(linux_acl.__salt__, {"acl.getfacl": mock}):
                # Update - test=True
                with patch.dict(linux_acl.__opts__, {"test": True}):
                    comt = (
                        "Updated permissions will be applied for {}: rwx -> {}".format(
                            acl_name, perms
                        )
                    )
                    ret = {
                        "name": name,
                        "comment": comt,
                        "changes": {
                            "new": {
                                "acl_name": acl_name,
                                "acl_type": acl_type,
                                "perms": perms,
                            },
                            "old": {
                                "acl_name": acl_name,
                                "acl_type": acl_type,
                                "perms": "rwx",
                            },
                        },
                        "result": None,
                    }

                    self.assertDictEqual(
                        linux_acl.present(
                            name, acl_type, acl_name, perms, recurse=True
                        ),
                        ret,
                    )

            # New - recurse true - nothing to do
            with patch.dict(linux_acl.__salt__, {"acl.getfacl": mock}):
                # Update - test=True
                with patch.dict(linux_acl.__opts__, {"test": True}):
                    comt = "Permissions are in the desired state"
                    ret = {"name": name, "comment": comt, "changes": {}, "result": True}

                    self.assertDictEqual(
                        linux_acl.present(
                            name, acl_type, acl_name, perms, recurse=True
                        ),
                        ret,
                    )

            # No acl type
            comt = "ACL Type does not exist"
            ret = {"name": name, "comment": comt, "result": False, "changes": {}}
            self.assertDictEqual(
                linux_acl.present(name, acl_type, acl_name, perms), ret
            )

            # default recurse false - nothing to do
            with patch.dict(linux_acl.__salt__, {"acl.getfacl": mock}):
                # Update - test=True
                with patch.dict(linux_acl.__opts__, {"test": True}):
                    comt = "Permissions are in the desired state"
                    ret = {"name": name, "comment": comt, "changes": {}, "result": True}

                    self.assertDictEqual(
                        linux_acl.present(
                            name, "d:" + acl_type, acl_name, perms, recurse=False
                        ),
                        ret,
                    )

            # default recurse false - nothing to do
            with patch.dict(linux_acl.__salt__, {"acl.getfacl": mock}):
                # Update - test=True
                with patch.dict(linux_acl.__opts__, {"test": True}):
                    comt = "Permissions are in the desired state"
                    ret = {"name": name, "comment": comt, "changes": {}, "result": True}

                    self.assertDictEqual(
                        linux_acl.present(
                            name, "d:" + acl_type, acl_name, perms, recurse=False
                        ),
                        ret,
                    )

            # default recurse true - nothing to do
            with patch.dict(linux_acl.__salt__, {"acl.getfacl": mock}):
                # Update - test=True
                with patch.dict(linux_acl.__opts__, {"test": True}):
                    comt = "Permissions are in the desired state"
                    ret = {"name": name, "comment": comt, "changes": {}, "result": True}

                    self.assertDictEqual(
                        linux_acl.present(
                            name, "d:" + acl_type, acl_name, perms, recurse=True
                        ),
                        ret,
                    )

    # 'absent' function tests: 2

    def test_absent(self):
        """
        Test to ensure a Linux ACL does not exist
        """
        name = "/root"
        acl_type = "users"
        acl_name = "damian"
        perms = "rwx"

        ret = {"name": name, "result": None, "comment": "", "changes": {}}

        mock = MagicMock(
            side_effect=[
                {name: {acl_type: [{acl_name: {"octal": "A"}}]}},
                {name: {acl_type: ""}},
            ]
        )
        with patch.dict(linux_acl.__salt__, {"acl.getfacl": mock}):
            with patch.dict(linux_acl.__opts__, {"test": True}):
                comt = "Removing permissions"
                ret.update({"comment": comt})
                self.assertDictEqual(
                    linux_acl.absent(name, acl_type, acl_name, perms), ret
                )

            comt = "ACL Type does not exist"
            ret.update({"comment": comt, "result": False})
            self.assertDictEqual(linux_acl.absent(name, acl_type, acl_name, perms), ret)

    # 'list_present' function tests: 1

    def test_list_present(self):
        """
        Test to ensure a Linux ACL is present
        """
        self.maxDiff = None
        name = "/root"
        acl_type = "user"
        acl_names = ["root", "damian", "homer"]
        acl_comment = {"owner": "root", "group": "root", "file": "/root"}
        perms = "rwx"

        mock = MagicMock(
            side_effect=[
                {
                    name: {
                        acl_type: [
                            {acl_names[0]: {"octal": "A"}},
                            {acl_names[1]: {"octal": "A"}},
                            {acl_names[2]: {"octal": "A"}},
                        ],
                        "comment": acl_comment,
                    }
                },
                {
                    name: {
                        acl_type: [
                            {acl_names[0]: {"octal": "A"}},
                            {acl_names[1]: {"octal": "A"}},
                        ],
                        "comment": acl_comment,
                    }
                },
                {
                    name: {
                        acl_type: [
                            {acl_names[0]: {"octal": "A"}},
                            {acl_names[1]: {"octal": "A"}},
                        ]
                    }
                },
                {name: {acl_type: [{}]}},
                {name: {acl_type: [{}]}},
                {name: {acl_type: [{}]}},
                {name: {acl_type: ""}},
            ]
        )
        mock_modfacl = MagicMock(return_value=True)

        with patch.dict(linux_acl.__salt__, {"acl.getfacl": mock}):
            # Update - test=True
            with patch.dict(linux_acl.__opts__, {"test": True}):
                comt = "Updated permissions will be applied for {}: A -> {}".format(
                    acl_names, perms
                )
                expected = {
                    "name": name,
                    "comment": comt,
                    "changes": {
                        "new": {
                            "acl_name": ", ".join(acl_names),
                            "acl_type": acl_type,
                            "perms": 7,
                        },
                        "old": {
                            "acl_name": ", ".join(acl_names),
                            "acl_type": acl_type,
                            "perms": "A",
                        },
                    },
                    "result": None,
                }

                ret = linux_acl.list_present(name, acl_type, acl_names, perms)
                self.assertDictEqual(ret, expected)

            # Update - test=False
            with patch.dict(linux_acl.__salt__, {"acl.modfacl": mock_modfacl}):
                with patch.dict(linux_acl.__opts__, {"test": False}):
                    comt = "Applied new permissions for {}".format(", ".join(acl_names))
                    expected = {
                        "name": name,
                        "comment": comt,
                        "changes": {
                            "new": {
                                "acl_name": ", ".join(acl_names),
                                "acl_type": acl_type,
                                "perms": "rwx",
                            }
                        },
                        "result": True,
                    }

                    ret = linux_acl.list_present(name, acl_type, acl_names, perms)
                    self.assertDictEqual(expected, ret)

            # Update - modfacl error
            with patch.dict(
                linux_acl.__salt__,
                {
                    "acl.modfacl": MagicMock(
                        side_effect=CommandExecutionError("Custom err")
                    )
                },
            ):
                with patch.dict(linux_acl.__opts__, {"test": False}):
                    comt = "Error updating permissions for {}: Custom err".format(
                        acl_names
                    )
                    expected = {
                        "name": name,
                        "comment": comt,
                        "changes": {},
                        "result": False,
                    }

                    ret = linux_acl.list_present(name, acl_type, acl_names, perms)
                    self.assertDictEqual(expected, ret)

            # New - test=True
            with patch.dict(linux_acl.__salt__, {"acl.modfacl": mock_modfacl}):
                with patch.dict(linux_acl.__opts__, {"test": True}):
                    comt = "New permissions will be applied for {}: {}".format(
                        acl_names, perms
                    )
                    expected = {
                        "name": name,
                        "comment": comt,
                        "changes": {
                            "new": {
                                "acl_name": ", ".join(acl_names),
                                "acl_type": acl_type,
                                "perms": perms,
                            }
                        },
                        "result": None,
                    }

                    ret = linux_acl.list_present(name, acl_type, acl_names, perms)
                    self.assertDictEqual(expected, ret)

            # New - test=False
            with patch.dict(linux_acl.__salt__, {"acl.modfacl": mock_modfacl}):
                with patch.dict(linux_acl.__opts__, {"test": False}):
                    comt = "Applied new permissions for {}".format(", ".join(acl_names))
                    expected = {
                        "name": name,
                        "comment": comt,
                        "changes": {
                            "new": {
                                "acl_name": ", ".join(acl_names),
                                "acl_type": acl_type,
                                "perms": perms,
                            }
                        },
                        "result": True,
                    }
                    ret = linux_acl.list_present(name, acl_type, acl_names, perms)
                    self.assertDictEqual(expected, ret)

            # New - modfacl error
            with patch.dict(
                linux_acl.__salt__,
                {
                    "acl.modfacl": MagicMock(
                        side_effect=CommandExecutionError("Custom err")
                    )
                },
            ):
                with patch.dict(linux_acl.__opts__, {"test": False}):
                    comt = "Error updating permissions for {}: Custom err".format(
                        acl_names
                    )
                    expected = {
                        "name": name,
                        "comment": comt,
                        "changes": {},
                        "result": False,
                    }

                    ret = linux_acl.list_present(name, acl_type, acl_names, perms)
                    self.assertDictEqual(expected, ret)

            # No acl type
            comt = "ACL Type does not exist"
            expected = {
                "name": name,
                "comment": comt,
                "result": False,
                "changes": {},
            }
            ret = linux_acl.list_present(name, acl_type, acl_names, perms)
            self.assertDictEqual(expected, ret)

    # 'list_absent' function tests: 2

    def test_list_absent(self):
        """
        Test to ensure a Linux ACL does not exist
        """
        name = "/root"
        acl_type = "users"
        acl_names = ["damian", "homer"]
        perms = "rwx"

        ret = {"name": name, "result": None, "comment": "", "changes": {}}

        mock = MagicMock(
            side_effect=[
                {
                    name: {
                        acl_type: [
                            {acl_names[0]: {"octal": "A"}, acl_names[1]: {"octal": "A"}}
                        ]
                    }
                },
                {name: {acl_type: ""}},
            ]
        )
        with patch.dict(linux_acl.__salt__, {"acl.getfacl": mock}):
            with patch.dict(linux_acl.__opts__, {"test": True}):
                comt = "Removing permissions"
                ret.update({"comment": comt})
                self.assertDictEqual(
                    linux_acl.list_absent(name, acl_type, acl_names, perms), ret
                )

            comt = "ACL Type does not exist"
            ret.update({"comment": comt, "result": False})
            self.assertDictEqual(linux_acl.list_absent(name, acl_type, acl_names), ret)

    def test_absent_recursive(self):
        """
        Test to ensure a Linux ACL does not exist
        """
        name = "/root"
        acl_type = "users"
        acl_name = "damian"
        perms = "rwx"

        ret = {"name": name, "result": None, "comment": "", "changes": {}}

        mock = MagicMock(
            side_effect=[
                {
                    name: {acl_type: [{acl_name: {"octal": 7}}]},
                    name + "/foo": {acl_type: [{acl_name: {"octal": "A"}}]},
                }
            ]
        )
        with patch.dict(linux_acl.__salt__, {"acl.getfacl": mock}):
            with patch.dict(linux_acl.__opts__, {"test": True}):
                comt = "Removing permissions"
                ret.update({"comment": comt})
                self.assertDictEqual(
                    linux_acl.absent(name, acl_type, acl_name, perms, recurse=True), ret
                )



import time

import attr
import pytest
from saltfactories.daemons.container import Container
from saltfactories.utils import random_string
from saltfactories.utils.ports import get_unused_localhost_port

docker = pytest.importorskip("docker")
docker_errors = pytest.importorskip("docker.errors")


@attr.s(kw_only=True, slots=True)
class MySQLImage:
    name = attr.ib()
    tag = attr.ib()
    container_id = attr.ib()

    def __str__(self):
        return "{}:{}".format(self.name, self.tag)


@attr.s(kw_only=True, slots=True)
class MySQLCombo:
    mysql_name = attr.ib()
    mysql_version = attr.ib()
    mysql_port = attr.ib()
    mysql_user = attr.ib()
    mysql_passwd = attr.ib()

    @mysql_port.default
    def _mysql_port(self):
        return get_unused_localhost_port()


def get_test_versions():
    test_versions = []
    name = "mysql/mysql-server"
    for version in ("5.5", "5.6", "5.7", "8.0"):
        test_versions.append(
            MySQLImage(
                name=name,
                tag=version,
                container_id=random_string("mysql-{}-".format(version)),
            )
        )
    name = "mariadb"
    for version in ("10.1", "10.2", "10.3", "10.4", "10.5"):
        test_versions.append(
            MySQLImage(
                name=name,
                tag=version,
                container_id=random_string("mariadb-{}-".format(version)),
            )
        )
    name = "percona"
    for version in ("5.5", "5.6", "5.7", "8.0"):
        test_versions.append(
            MySQLImage(
                name=name,
                tag=version,
                container_id=random_string("percona-{}-".format(version)),
            )
        )
    return test_versions


def get_test_version_id(value):
    return "container={}".format(value)


@pytest.fixture(scope="module", params=get_test_versions(), ids=get_test_version_id)
def mysql_container(request, salt_factories, salt_call_cli):

    try:
        docker_client = docker.from_env()
    except docker_errors.DockerException:
        pytest.skip("Failed to get a connection to docker running on the system")
    connectable = Container.client_connectable(docker_client)
    if connectable is not True:  # pragma: no cover
        pytest.skip(connectable)

    mysql_image = request.param

    mysql_user = "root"
    mysql_passwd = "password"

    combo = MySQLCombo(
        mysql_name=mysql_image.name,
        mysql_version=mysql_image.tag,
        mysql_user=mysql_user,
        mysql_passwd=mysql_passwd,
    )
    container = salt_factories.get_container(
        mysql_image.container_id,
        "{}:{}".format(combo.mysql_name, combo.mysql_version),
        docker_client=docker_client,
        check_ports=[combo.mysql_port],
        container_run_kwargs={
            "ports": {"3306/tcp": combo.mysql_port},
            "environment": {
                "MYSQL_ROOT_PASSWORD": mysql_passwd,
                "MYSQL_ROOT_HOST": "%",
            },
        },
    )
    with container.started():
        authenticated = False
        login_attempts = 6
        while login_attempts:
            login_attempts -= 1
            # Make sure "MYSQL" is ready
            ret = salt_call_cli.run(
                "docker.run",
                name=mysql_image.container_id,
                cmd="mysql --user=root --password=password -e 'SELECT 1'",
            )
            authenticated = ret.exitcode == 0
            if authenticated:
                break

            time.sleep(2)

        if authenticated:
            yield combo
        else:
            pytest.fail(
                "Failed to login into mysql server running in container(id: {})".format(
                    mysql_image.container_id
                )
            )



"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""

import pytest
import salt.states.memcached as memcached
from salt.exceptions import CommandExecutionError
from tests.support.mock import MagicMock, patch


@pytest.fixture
def configure_loader_modules():
    return {memcached: {}}


def test_managed():
    """
    Test to manage a memcached key.
    """
    name = "foo"

    ret = {"name": name, "result": False, "comment": "", "changes": {}}

    mock_t = MagicMock(side_effect=[CommandExecutionError, "salt", True, True, True])
    with patch.dict(
        memcached.__salt__, {"memcached.get": mock_t, "memcached.set": mock_t}
    ):
        assert memcached.managed(name) == ret

        comt = "Key 'foo' does not need to be updated"
        ret.update({"comment": comt, "result": True})
        assert memcached.managed(name, "salt") == ret

        with patch.dict(memcached.__opts__, {"test": True}):
            comt = "Value of key 'foo' would be changed"
            ret.update({"comment": comt, "result": None})
            assert memcached.managed(name, "salt") == ret

        with patch.dict(memcached.__opts__, {"test": False}):
            comt = "Successfully set key 'foo'"
            ret.update(
                {
                    "comment": comt,
                    "result": True,
                    "changes": {"new": "salt", "old": True},
                }
            )
            assert memcached.managed(name, "salt") == ret


def test_absent():
    """
    Test to ensure that a memcached key is not present.
    """
    name = "foo"

    ret = {"name": name, "result": False, "comment": "", "changes": {}}

    mock_t = MagicMock(
        side_effect=[CommandExecutionError, "salt", None, True, True, True]
    )
    with patch.dict(
        memcached.__salt__, {"memcached.get": mock_t, "memcached.delete": mock_t}
    ):
        assert memcached.absent(name) == ret

        comt = "Value of key 'foo' ('salt') is not 'bar'"
        ret.update({"comment": comt, "result": True})
        assert memcached.absent(name, "bar") == ret

        comt = "Key 'foo' does not exist"
        ret.update({"comment": comt})
        assert memcached.absent(name) == ret

        with patch.dict(memcached.__opts__, {"test": True}):
            comt = "Key 'foo' would be deleted"
            ret.update({"comment": comt, "result": None})
            assert memcached.absent(name) == ret

        with patch.dict(memcached.__opts__, {"test": False}):
            comt = "Successfully deleted key 'foo'"
            ret.update(
                {
                    "comment": comt,
                    "result": True,
                    "changes": {"key deleted": "foo", "value": True},
                }
            )
            assert memcached.absent(name) == ret



import salt.modules.artifactory as artifactory
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class ArtifactoryTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        return {artifactory: {}}

    def test_artifact_get_metadata(self):
        with patch(
            "salt.modules.artifactory._get_artifact_metadata_xml",
            MagicMock(
                return_value="""<?xml version="1.0" encoding="UTF-8"?>
                <metadata>
                  <groupId>com.company.sampleapp.web-module</groupId>
                  <artifactId>web</artifactId>
                  <versioning>
                    <latest>1.1_RC11</latest>
                    <release>1.0.1</release>
                    <versions>
                      <version>1.0_RC20</version>
                      <version>1.0_RC22</version>
                    </versions>
                    <lastUpdated>20140623120632</lastUpdated>
                  </versioning>
                </metadata>
            """
            ),
        ):
            metadata = artifactory._get_artifact_metadata(
                artifactory_url="http://artifactory.example.com/artifactory",
                repository="libs-releases",
                group_id="com.company.sampleapp.web-module",
                artifact_id="web",
                headers={},
            )
            self.assertEqual(metadata["latest_version"], "1.1_RC11")

    def test_snapshot_version_get_metadata(self):
        with patch(
            "salt.modules.artifactory._get_snapshot_version_metadata_xml",
            MagicMock(
                return_value="""<?xml version="1.0" encoding="UTF-8"?>
                    <metadata>
                      <groupId>com.company.sampleapp.web-module</groupId>
                      <artifactId>web</artifactId>
                      <version>1.1_RC8-SNAPSHOT</version>
                      <versioning>
                        <snapshot>
                          <timestamp>20140418.150212</timestamp>
                          <buildNumber>1</buildNumber>
                        </snapshot>
                        <lastUpdated>20140623104055</lastUpdated>
                        <snapshotVersions>
                          <snapshotVersion>
                            <extension>pom</extension>
                            <value>1.1_RC8-20140418.150212-1</value>
                            <updated>20140418150212</updated>
                          </snapshotVersion>
                          <snapshotVersion>
                            <extension>war</extension>
                            <value>1.1_RC8-20140418.150212-1</value>
                            <updated>20140418150212</updated>
                          </snapshotVersion>
                        </snapshotVersions>
                      </versioning>
                    </metadata>
                """
            ),
        ):
            metadata = artifactory._get_snapshot_version_metadata(
                artifactory_url="http://artifactory.example.com/artifactory",
                repository="libs-releases",
                group_id="com.company.sampleapp.web-module",
                artifact_id="web",
                version="1.1_RC8-SNAPSHOT",
                headers={},
            )
            self.assertEqual(
                metadata["snapshot_versions"]["war"], "1.1_RC8-20140418.150212-1"
            )

    def test_artifact_metadata_url(self):
        metadata_url = artifactory._get_artifact_metadata_url(
            artifactory_url="http://artifactory.example.com/artifactory",
            repository="libs-releases",
            group_id="com.company.sampleapp.web-module",
            artifact_id="web",
        )

        self.assertEqual(
            metadata_url,
            "http://artifactory.example.com/artifactory/libs-releases/com/company/sampleapp/web-module/web/maven-metadata.xml",
        )

    def test_snapshot_version_metadata_url(self):
        metadata_url = artifactory._get_snapshot_version_metadata_url(
            artifactory_url="http://artifactory.example.com/artifactory",
            repository="libs-snapshots",
            group_id="com.company.sampleapp.web-module",
            artifact_id="web",
            version="1.0_RC10-SNAPSHOT",
        )

        self.assertEqual(
            metadata_url,
            "http://artifactory.example.com/artifactory/libs-snapshots/com/company/sampleapp/web-module/web/1.0_RC10-SNAPSHOT/maven-metadata.xml",
        )

    def test_construct_url_for_released_version(self):
        artifact_url, file_name = artifactory._get_release_url(
            repository="libs-releases",
            group_id="com.company.sampleapp.web-module",
            artifact_id="web",
            packaging="war",
            artifactory_url="http://artifactory.example.com/artifactory",
            version="1.0_RC20",
        )

        self.assertEqual(
            artifact_url,
            "http://artifactory.example.com/artifactory/libs-releases/com/company/sampleapp/web-module/web/1.0_RC20/web-1.0_RC20.war",
        )
        self.assertEqual(file_name, "web-1.0_RC20.war")

    def test_construct_url_for_snapshot_version(self):
        with patch(
            "salt.modules.artifactory._get_snapshot_version_metadata",
            MagicMock(
                return_value={
                    "snapshot_versions": {"war": "1.0_RC10-20131127.105838-2"}
                }
            ),
        ):

            artifact_url, file_name = artifactory._get_snapshot_url(
                artifactory_url="http://artifactory.example.com/artifactory",
                repository="libs-snapshots",
                group_id="com.company.sampleapp.web-module",
                artifact_id="web",
                version="1.0_RC10-SNAPSHOT",
                packaging="war",
                headers={},
            )

            self.assertEqual(
                artifact_url,
                "http://artifactory.example.com/artifactory/libs-snapshots/com/company/sampleapp/web-module/web/1.0_RC10-SNAPSHOT/web-1.0_RC10-20131127.105838-2.war",
            )
            self.assertEqual(file_name, "web-1.0_RC10-20131127.105838-2.war")

    def test_get_snapshot_url_with_classifier(self):
        with patch(
            "salt.modules.artifactory._get_snapshot_version_metadata_xml",
            MagicMock(
                return_value="""<?xml version="1.0" encoding="UTF-8"?>
                    <metadata>
                      <groupId>com.company.sampleapp.web-module</groupId>
                      <artifactId>web</artifactId>
                      <version>1.1_RC8-SNAPSHOT</version>
                      <versioning>
                        <snapshot>
                          <timestamp>20140418.150212</timestamp>
                          <buildNumber>1</buildNumber>
                        </snapshot>
                        <lastUpdated>20140623104055</lastUpdated>
                        <snapshotVersions>
                          <snapshotVersion>
                            <extension>pom</extension>
                            <value>1.1_RC8-20140418.150212-1</value>
                            <updated>20140418150212</updated>
                          </snapshotVersion>
                          <snapshotVersion>
                            <classifier>test</classifier>
                            <extension>war</extension>
                            <value>1.1_RC8-20140418.150212-1</value>
                            <updated>20140418150212</updated>
                          </snapshotVersion>
                        </snapshotVersions>
                      </versioning>
                    </metadata>
                """
            ),
        ):
            artifact_url, file_name = artifactory._get_snapshot_url(
                artifactory_url="http://artifactory.example.com/artifactory",
                repository="libs-snapshots",
                group_id="com.company.sampleapp.web-module",
                artifact_id="web",
                version="1.1_RC8-SNAPSHOT",
                packaging="war",
                classifier="test",
                headers={},
            )

            self.assertEqual(
                artifact_url,
                "http://artifactory.example.com/artifactory/libs-snapshots/com/company/sampleapp/web-module/web/1.1_RC8-SNAPSHOT/web-1.1_RC8-20140418.150212-1-test.war",
            )

    def test_get_snapshot_url_without_classifier(self):
        """
        test when classifier not set and packaging
        does not match snapshot_versions in the metadata.
        """
        with patch(
            "salt.modules.artifactory._get_snapshot_version_metadata_xml",
            MagicMock(
                return_value="""<?xml version="1.0" encoding="UTF-8"?>
                    <metadata>
                      <groupId>com.company.sampleapp.web-module</groupId>
                      <artifactId>web</artifactId>
                      <version>1.1_RC8-SNAPSHOT</version>
                      <versioning>
                        <snapshot>
                          <timestamp>20140418.150212</timestamp>
                          <buildNumber>1</buildNumber>
                        </snapshot>
                        <lastUpdated>20140623104055</lastUpdated>
                        <snapshotVersions>
                          <snapshotVersion>
                            <extension>pom</extension>
                            <value>1.1_RC8-20140418.150212-1</value>
                            <updated>20140418150212</updated>
                          </snapshotVersion>
                          <snapshotVersion>
                            <classifier>test</classifier>
                            <extension>war</extension>
                            <value>1.1_RC8-20140418.150212-1</value>
                            <updated>20140418150212</updated>
                          </snapshotVersion>
                        </snapshotVersions>
                      </versioning>
                    </metadata>
                """
            ),
        ):
            with self.assertRaises(artifactory.ArtifactoryError):
                artifact_url, file_name = artifactory._get_snapshot_url(
                    artifactory_url="http://artifactory.example.com/artifactory",
                    repository="libs-snapshots",
                    group_id="com.company.sampleapp.web-module",
                    artifact_id="web",
                    version="1.1_RC8-SNAPSHOT",
                    packaging="war",
                    headers={},
                )

    def test_get_latest_snapshot_username_password(self):
        with patch(
            "salt.modules.artifactory._get_artifact_metadata",
            return_value={"latest_version": "1.1"},
        ), patch(
            "salt.modules.artifactory._get_snapshot_url",
            return_value=(
                "http://artifactory.example.com/artifactory/snapshot",
                "/path/to/file",
            ),
        ), patch(
            "salt.modules.artifactory.__save_artifact", return_value={}
        ) as save_artifact_mock:
            artifactory.get_latest_snapshot(
                artifactory_url="http://artifactory.example.com/artifactory",
                repository="libs-snapshots",
                group_id="com.company.sampleapp.web-module",
                artifact_id="web",
                packaging="war",
                username="user",
                password="password",
            )
            save_artifact_mock.assert_called_with(
                "http://artifactory.example.com/artifactory/snapshot",
                "/path/to/file",
                {"Authorization": "Basic dXNlcjpwYXNzd29yZA==\n"},
            )

    def test_get_snapshot_username_password(self):
        with patch(
            "salt.modules.artifactory._get_snapshot_url",
            return_value=(
                "http://artifactory.example.com/artifactory/snapshot",
                "/path/to/file",
            ),
        ), patch(
            "salt.modules.artifactory.__save_artifact", return_value={}
        ) as save_artifact_mock:
            artifactory.get_snapshot(
                artifactory_url="http://artifactory.example.com/artifactory",
                repository="libs-snapshots",
                group_id="com.company.sampleapp.web-module",
                artifact_id="web",
                packaging="war",
                version="1.1",
                username="user",
                password="password",
            )
            save_artifact_mock.assert_called_with(
                "http://artifactory.example.com/artifactory/snapshot",
                "/path/to/file",
                {"Authorization": "Basic dXNlcjpwYXNzd29yZA==\n"},
            )

    def test_get_latest_release_username_password(self):
        with patch(
            "salt.modules.artifactory.__find_latest_version",
            return_value="1.1",
        ), patch(
            "salt.modules.artifactory._get_release_url",
            return_value=(
                "http://artifactory.example.com/artifactory/release",
                "/path/to/file",
            ),
        ), patch(
            "salt.modules.artifactory.__save_artifact", return_value={}
        ) as save_artifact_mock:
            artifactory.get_latest_release(
                artifactory_url="http://artifactory.example.com/artifactory",
                repository="libs-snapshots",
                group_id="com.company.sampleapp.web-module",
                artifact_id="web",
                packaging="war",
                username="user",
                password="password",
            )
            save_artifact_mock.assert_called_with(
                "http://artifactory.example.com/artifactory/release",
                "/path/to/file",
                {"Authorization": "Basic dXNlcjpwYXNzd29yZA==\n"},
            )

    def test_get_release_username_password(self):
        with patch(
            "salt.modules.artifactory._get_release_url",
            return_value=(
                "http://artifactory.example.com/artifactory/release",
                "/path/to/file",
            ),
        ), patch(
            "salt.modules.artifactory.__save_artifact", return_value={}
        ) as save_artifact_mock:
            artifactory.get_release(
                artifactory_url="http://artifactory.example.com/artifactory",
                repository="libs-snapshots",
                group_id="com.company.sampleapp.web-module",
                artifact_id="web",
                packaging="war",
                version="1.1",
                username="user",
                password="password",
            )
            save_artifact_mock.assert_called_with(
                "http://artifactory.example.com/artifactory/release",
                "/path/to/file",
                {"Authorization": "Basic dXNlcjpwYXNzd29yZA==\n"},
            )

    def test_save_artifact_file_exists_checksum_equal(self):
        artifact_url = "http://artifactory.example.com/artifactory/artifact"
        target_file = "/path/to/file"
        sum_str = "0123456789abcdef0123456789abcdef01234567"
        sum_bin = sum_str.encode()
        with patch("os.path.isfile", return_value=True), patch.dict(
            artifactory.__salt__, {"file.get_hash": MagicMock(return_value=sum_str)}
        ):
            with patch(
                "salt.modules.artifactory.__download",
                return_value=(True, sum_bin, None),
            ):
                result = getattr(artifactory, "__save_artifact")(
                    artifact_url=artifact_url, target_file=target_file, headers={}
                )
                assert result == {
                    "status": True,
                    "changes": {},
                    "target_file": target_file,
                    "comment": (
                        "File {} already exists, checksum matches with Artifactory.\n"
                        "Checksum URL: {}.sha1".format(target_file, artifact_url)
                    ),
                }
            with patch(
                "salt.modules.artifactory.__download",
                return_value=(True, sum_str, None),
            ):
                result = getattr(artifactory, "__save_artifact")(
                    artifact_url=artifact_url, target_file=target_file, headers={}
                )
                assert result == {
                    "status": True,
                    "changes": {},
                    "target_file": target_file,
                    "comment": (
                        "File {} already exists, checksum matches with Artifactory.\n"
                        "Checksum URL: {}.sha1".format(target_file, artifact_url)
                    ),
                }



"""
Management of OpenStack Glance Images
========================================

.. versionadded:: 2018.3.0

:depends: shade
:configuration: see :py:mod:`salt.modules.glanceng` for setup instructions

Example States

.. code-block:: yaml

    create image:
      glance_image.present:
        - name: cirros
        - filename: cirros.raw
        - image_format: raw

    delete image:
      glance_image.absent:
        - name: cirros
"""


__virtualname__ = "glance_image"


def __virtual__():
    if "glanceng.image_get" in __salt__:
        return __virtualname__
    return (
        False,
        "The glanceng execution module failed to load: shade python module is not"
        " available",
    )


def present(name, auth=None, **kwargs):
    """
    Ensure image exists and is up-to-date

    name
        Name of the image

    enabled
        Boolean to control if image is enabled

    description
        An arbitrary description of the image
    """
    ret = {"name": name, "changes": {}, "result": True, "comment": ""}

    kwargs = __utils__["args.clean_kwargs"](**kwargs)

    __salt__["glanceng.setup_clouds"](auth)

    image = __salt__["glanceng.image_get"](name=name)

    if not image:
        if __opts__["test"]:
            ret["result"] = None
            ret["changes"] = kwargs
            ret["comment"] = "Image {} will be created.".format(name)
            return ret

        kwargs["name"] = name
        image = __salt__["glanceng.image_create"](**kwargs)
        ret["changes"] = image
        ret["comment"] = "Created image"
        return ret

    # TODO(SamYaple): Compare and update image properties here
    return ret


def absent(name, auth=None):
    """
    Ensure image does not exist

    name
        Name of the image
    """
    ret = {"name": name, "changes": {}, "result": True, "comment": ""}

    __salt__["glanceng.setup_clouds"](auth)

    image = __salt__["glanceng.image_get"](name=name)

    if image:
        if __opts__["test"]:
            ret["result"] = None
            ret["changes"] = {"name": name}
            ret["comment"] = "Image {} will be deleted.".format(name)
            return ret

        __salt__["glanceng.image_delete"](name=image)
        ret["changes"]["id"] = image.id
        ret["comment"] = "Deleted image"

    return ret



import salt.states.helm as helm
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class HelmTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.helm
    """

    def setup_loader_modules(self):
        return {helm: {}}

    def test_repo_managed_import_failed_repo_manage(self):
        ret = {
            "name": "state_id",
            "changes": {},
            "result": False,
            "comment": "'helm.repo_manage' modules not available on this minion.",
        }
        self.assertEqual(helm.repo_managed("state_id"), ret)

    def test_repo_managed_import_failed_repo_update(self):
        mock_helm_modules = {"helm.repo_manage": MagicMock(return_value=True)}
        with patch.dict(helm.__salt__, mock_helm_modules):
            ret = {
                "name": "state_id",
                "changes": {},
                "result": False,
                "comment": "'helm.repo_update' modules not available on this minion.",
            }
            self.assertEqual(helm.repo_managed("state_id"), ret)

    def test_repo_managed_is_testing(self):
        mock_helm_modules = {
            "helm.repo_manage": MagicMock(return_value=True),
            "helm.repo_update": MagicMock(return_value=True),
        }
        with patch.dict(helm.__salt__, mock_helm_modules):
            mock__opts__ = {"test": MagicMock(return_value=True)}
            with patch.dict(helm.__opts__, mock__opts__):
                ret = {
                    "name": "state_id",
                    "result": None,
                    "comment": "Helm repo would have been managed.",
                    "changes": {},
                }
                self.assertEqual(helm.repo_managed("state_id"), ret)

    def test_repo_managed_success(self):
        result_changes = {"added": True, "removed": True, "failed": False}
        mock_helm_modules = {
            "helm.repo_manage": MagicMock(return_value=result_changes),
            "helm.repo_update": MagicMock(return_value=True),
        }
        with patch.dict(helm.__salt__, mock_helm_modules):
            ret = {
                "name": "state_id",
                "result": True,
                "comment": "Repositories were added or removed.",
                "changes": result_changes,
            }
            self.assertEqual(helm.repo_managed("state_id"), ret)

    def test_repo_managed_success_with_update(self):
        result_changes = {"added": True, "removed": True, "failed": False}
        mock_helm_modules = {
            "helm.repo_manage": MagicMock(return_value=result_changes),
            "helm.repo_update": MagicMock(return_value=True),
        }
        result_wanted = result_changes
        result_wanted.update({"repo_update": True})
        with patch.dict(helm.__salt__, mock_helm_modules):
            ret = {
                "name": "state_id",
                "result": True,
                "comment": "Repositories were added or removed.",
                "changes": result_wanted,
            }
            self.assertEqual(helm.repo_managed("state_id"), ret)

    def test_repo_managed_failed(self):
        result_changes = {"added": True, "removed": True, "failed": True}
        mock_helm_modules = {
            "helm.repo_manage": MagicMock(return_value=result_changes),
            "helm.repo_update": MagicMock(return_value=True),
        }
        with patch.dict(helm.__salt__, mock_helm_modules):
            ret = {
                "name": "state_id",
                "result": False,
                "comment": "Failed to add or remove some repositories.",
                "changes": result_changes,
            }
            self.assertEqual(helm.repo_managed("state_id"), ret)

    def test_repo_updated_import_failed(self):
        ret = {
            "name": "state_id",
            "changes": {},
            "result": False,
            "comment": "'helm.repo_update' modules not available on this minion.",
        }
        self.assertEqual(helm.repo_updated("state_id"), ret)

    def test_repo_updated_is_testing(self):
        mock_helm_modules = {"helm.repo_update": MagicMock(return_value=True)}
        with patch.dict(helm.__salt__, mock_helm_modules):
            mock__opts__ = {"test": MagicMock(return_value=True)}
            with patch.dict(helm.__opts__, mock__opts__):
                ret = {
                    "name": "state_id",
                    "result": None,
                    "comment": "Helm repo would have been updated.",
                    "changes": {},
                }
                self.assertEqual(helm.repo_updated("state_id"), ret)

    def test_repo_updated_success(self):
        mock_helm_modules = {"helm.repo_update": MagicMock(return_value=True)}
        with patch.dict(helm.__salt__, mock_helm_modules):
            ret = {
                "name": "state_id",
                "result": True,
                "comment": "Helm repo is updated.",
                "changes": {},
            }
            self.assertEqual(helm.repo_updated("state_id"), ret)

    def test_repo_updated_failed(self):
        mock_helm_modules = {"helm.repo_update": MagicMock(return_value=False)}
        with patch.dict(helm.__salt__, mock_helm_modules):
            ret = {
                "name": "state_id",
                "result": False,
                "comment": "Failed to sync some repositories.",
                "changes": False,
            }
            self.assertEqual(helm.repo_updated("state_id"), ret)

    def test_release_present_import_failed_helm_status(self):
        ret = {
            "name": "state_id",
            "changes": {},
            "result": False,
            "comment": "'helm.status' modules not available on this minion.",
        }
        self.assertEqual(helm.release_present("state_id", "mychart"), ret)

    def test_release_present_import_failed_helm_install(self):
        mock_helm_modules = {"helm.status": MagicMock(return_value=True)}
        with patch.dict(helm.__salt__, mock_helm_modules):
            ret = {
                "name": "state_id",
                "changes": {},
                "result": False,
                "comment": "'helm.install' modules not available on this minion.",
            }
            self.assertEqual(helm.release_present("state_id", "mychart"), ret)

    def test_release_present_import_failed_helm_upgrade(self):
        mock_helm_modules = {
            "helm.status": MagicMock(return_value=True),
            "helm.install": MagicMock(return_value=True),
        }
        with patch.dict(helm.__salt__, mock_helm_modules):
            ret = {
                "name": "state_id",
                "changes": {},
                "result": False,
                "comment": "'helm.upgrade' modules not available on this minion.",
            }
            self.assertEqual(helm.release_present("state_id", "mychart"), ret)

    def test_release_present_is_testing(self):
        mock_helm_modules = {
            "helm.status": MagicMock(return_value=True),
            "helm.install": MagicMock(return_value=True),
            "helm.upgrade": MagicMock(return_value=True),
        }
        with patch.dict(helm.__salt__, mock_helm_modules):
            mock__opts__ = {"test": MagicMock(return_value=True)}
            with patch.dict(helm.__opts__, mock__opts__):
                ret = {
                    "name": "state_id",
                    "result": None,
                    "comment": "Helm release would have been installed or updated.",
                    "changes": {},
                }
                self.assertEqual(helm.release_present("state_id", "mychart"), ret)

    def test_release_absent_import_failed_helm_uninstall(self):
        ret = {
            "name": "state_id",
            "changes": {},
            "result": False,
            "comment": "'helm.uninstall' modules not available on this minion.",
        }
        self.assertEqual(helm.release_absent("state_id"), ret)

    def test_release_absent_import_failed_helm_status(self):
        mock_helm_modules = {"helm.uninstall": MagicMock(return_value=True)}
        with patch.dict(helm.__salt__, mock_helm_modules):
            ret = {
                "name": "state_id",
                "changes": {},
                "result": False,
                "comment": "'helm.status' modules not available on this minion.",
            }
            self.assertEqual(helm.release_absent("state_id"), ret)

    def test_release_absent_is_testing(self):
        mock_helm_modules = {
            "helm.status": MagicMock(return_value=True),
            "helm.uninstall": MagicMock(return_value=True),
        }
        with patch.dict(helm.__salt__, mock_helm_modules):
            mock__opts__ = {"test": MagicMock(return_value=True)}
            with patch.dict(helm.__opts__, mock__opts__):
                ret = {
                    "name": "state_id",
                    "result": None,
                    "comment": "Helm release would have been uninstalled.",
                    "changes": {},
                }
                self.assertEqual(helm.release_absent("state_id"), ret)

    def test_release_absent_success(self):
        mock_helm_modules = {
            "helm.status": MagicMock(return_value={}),
            "helm.uninstall": MagicMock(return_value=True),
        }
        with patch.dict(helm.__salt__, mock_helm_modules):
            ret = {
                "name": "state_id",
                "result": True,
                "comment": "Helm release state_id is absent.",
                "changes": {"absent": "state_id"},
            }
            self.assertEqual(helm.release_absent("state_id"), ret)

    def test_release_absent_error(self):
        mock_helm_modules = {
            "helm.status": MagicMock(return_value={}),
            "helm.uninstall": MagicMock(return_value="error"),
        }
        with patch.dict(helm.__salt__, mock_helm_modules):
            ret = {
                "name": "state_id",
                "result": False,
                "comment": "error",
                "changes": {},
            }
            self.assertEqual(helm.release_absent("state_id"), ret)



#
# Author: Alberto Planas <aplanas@suse.com>
#
# Copyright 2018 SUSE LINUX GmbH, Nuernberg, Germany.
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

"""
:maintainer:    Alberto Planas <aplanas@suse.com>
:platform:      Linux
"""

import sys

import salt.modules.chroot as chroot
import salt.utils.platform
from salt.exceptions import CommandExecutionError
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase, skipIf


@skipIf(salt.utils.platform.is_windows(), "This test cannot work on Windows")
class ChrootTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.chroot
    """

    def setup_loader_modules(self):
        return {chroot: {"__salt__": {}, "__utils__": {}, "__opts__": {"cachedir": ""}}}

    @patch("os.path.isdir")
    def test_exist(self, isdir):
        """
        Test if the chroot environment exist.
        """
        isdir.side_effect = (True, True, True, True)
        self.assertTrue(chroot.exist("/chroot"))

        isdir.side_effect = (True, True, True, False)
        self.assertFalse(chroot.exist("/chroot"))

    @patch("os.makedirs")
    @patch("salt.modules.chroot.exist")
    def test_create(self, exist, makedirs):
        """
        Test the creation of an empty chroot environment.
        """
        exist.return_value = True
        self.assertTrue(chroot.create("/chroot"))
        makedirs.assert_not_called()

        exist.return_value = False
        self.assertTrue(chroot.create("/chroot"))
        makedirs.assert_called()

    @patch("salt.utils.files.fopen")
    def test_in_chroot(self, fopen):
        """
        Test the detection of chroot environment.
        """
        matrix = (("a", "b", True), ("a", "a", False))
        for root_mountinfo, self_mountinfo, result in matrix:
            fopen.return_value.__enter__.return_value = fopen
            fopen.read = MagicMock(side_effect=(root_mountinfo, self_mountinfo))
            self.assertEqual(chroot.in_chroot(), result)

    @patch("salt.modules.chroot.exist")
    def test_call_fails_input_validation(self, exist):
        """
        Test execution of Salt functions in chroot.
        """
        # Basic input validation
        exist.return_value = False
        self.assertRaises(CommandExecutionError, chroot.call, "/chroot", "")
        self.assertRaises(CommandExecutionError, chroot.call, "/chroot", "test.ping")

    @patch("salt.modules.chroot.exist")
    @patch("tempfile.mkdtemp")
    def test_call_fails_untar(self, mkdtemp, exist):
        """
        Test execution of Salt functions in chroot.
        """
        # Fail the tar command
        exist.return_value = True
        mkdtemp.return_value = "/chroot/tmp01"
        utils_mock = {
            "thin.gen_thin": MagicMock(return_value="/salt-thin.tgz"),
            "files.rm_rf": MagicMock(),
        }
        salt_mock = {
            "cmd.run": MagicMock(return_value="Error"),
            "config.option": MagicMock(),
        }
        with patch.dict(chroot.__utils__, utils_mock), patch.dict(
            chroot.__salt__, salt_mock
        ):
            self.assertEqual(
                chroot.call("/chroot", "test.ping"),
                {"result": False, "comment": "Error"},
            )
            utils_mock["thin.gen_thin"].assert_called_once()
            salt_mock["config.option"].assert_called()
            salt_mock["cmd.run"].assert_called_once()
            utils_mock["files.rm_rf"].assert_called_once()

    @patch("salt.modules.chroot.exist")
    @patch("tempfile.mkdtemp")
    def test_call_fails_salt_thin(self, mkdtemp, exist):
        """
        Test execution of Salt functions in chroot.
        """
        # Fail the inner command
        exist.return_value = True
        mkdtemp.return_value = "/chroot/tmp01"
        utils_mock = {
            "thin.gen_thin": MagicMock(return_value="/salt-thin.tgz"),
            "files.rm_rf": MagicMock(),
            "json.find_json": MagicMock(side_effect=ValueError()),
        }
        salt_mock = {
            "cmd.run": MagicMock(return_value=""),
            "config.option": MagicMock(),
            "cmd.run_chroot": MagicMock(
                return_value={"retcode": 1, "stdout": "", "stderr": "Error"}
            ),
        }
        with patch.dict(chroot.__utils__, utils_mock), patch.dict(
            chroot.__salt__, salt_mock
        ):
            self.assertEqual(
                chroot.call("/chroot", "test.ping"),
                {
                    "result": False,
                    "retcode": 1,
                    "comment": {"stdout": "", "stderr": "Error"},
                },
            )
            utils_mock["thin.gen_thin"].assert_called_once()
            salt_mock["config.option"].assert_called()
            salt_mock["cmd.run"].assert_called_once()
            salt_mock["cmd.run_chroot"].assert_called_with(
                "/chroot",
                [
                    "python{}".format(sys.version_info[0]),
                    "/tmp01/salt-call",
                    "--metadata",
                    "--local",
                    "--log-file",
                    "/tmp01/log",
                    "--cachedir",
                    "/tmp01/cache",
                    "--out",
                    "json",
                    "-l",
                    "quiet",
                    "--",
                    "test.ping",
                ],
            )
            utils_mock["files.rm_rf"].assert_called_once()

    @patch("salt.modules.chroot.exist")
    @patch("tempfile.mkdtemp")
    def test_call_success(self, mkdtemp, exist):
        """
        Test execution of Salt functions in chroot.
        """
        # Success test
        exist.return_value = True
        mkdtemp.return_value = "/chroot/tmp01"
        utils_mock = {
            "thin.gen_thin": MagicMock(return_value="/salt-thin.tgz"),
            "files.rm_rf": MagicMock(),
            "json.find_json": MagicMock(return_value={"return": "result"}),
        }
        salt_mock = {
            "cmd.run": MagicMock(return_value=""),
            "config.option": MagicMock(),
            "cmd.run_chroot": MagicMock(return_value={"retcode": 0, "stdout": ""}),
        }
        with patch.dict(chroot.__utils__, utils_mock), patch.dict(
            chroot.__salt__, salt_mock
        ):
            self.assertEqual(chroot.call("/chroot", "test.ping"), "result")
            utils_mock["thin.gen_thin"].assert_called_once()
            salt_mock["config.option"].assert_called()
            salt_mock["cmd.run"].assert_called_once()
            salt_mock["cmd.run_chroot"].assert_called_with(
                "/chroot",
                [
                    "python{}".format(sys.version_info[0]),
                    "/tmp01/salt-call",
                    "--metadata",
                    "--local",
                    "--log-file",
                    "/tmp01/log",
                    "--cachedir",
                    "/tmp01/cache",
                    "--out",
                    "json",
                    "-l",
                    "quiet",
                    "--",
                    "test.ping",
                ],
            )
            utils_mock["files.rm_rf"].assert_called_once()

    @patch("salt.modules.chroot.exist")
    @patch("tempfile.mkdtemp")
    def test_call_success_parameters(self, mkdtemp, exist):
        """
        Test execution of Salt functions in chroot with parameters.
        """
        # Success test
        exist.return_value = True
        mkdtemp.return_value = "/chroot/tmp01"
        utils_mock = {
            "thin.gen_thin": MagicMock(return_value="/salt-thin.tgz"),
            "files.rm_rf": MagicMock(),
            "json.find_json": MagicMock(return_value={"return": "result"}),
        }
        salt_mock = {
            "cmd.run": MagicMock(return_value=""),
            "config.option": MagicMock(),
            "cmd.run_chroot": MagicMock(return_value={"retcode": 0, "stdout": ""}),
        }
        with patch.dict(chroot.__utils__, utils_mock), patch.dict(
            chroot.__salt__, salt_mock
        ):
            self.assertEqual(
                chroot.call("/chroot", "module.function", key="value"), "result"
            )
            utils_mock["thin.gen_thin"].assert_called_once()
            salt_mock["config.option"].assert_called()
            salt_mock["cmd.run"].assert_called_once()
            salt_mock["cmd.run_chroot"].assert_called_with(
                "/chroot",
                [
                    "python{}".format(sys.version_info[0]),
                    "/tmp01/salt-call",
                    "--metadata",
                    "--local",
                    "--log-file",
                    "/tmp01/log",
                    "--cachedir",
                    "/tmp01/cache",
                    "--out",
                    "json",
                    "-l",
                    "quiet",
                    "--",
                    "module.function",
                    "key=value",
                ],
            )
            utils_mock["files.rm_rf"].assert_called_once()

    @patch("salt.modules.chroot._create_and_execute_salt_state")
    @patch("salt.client.ssh.state.SSHHighState")
    @patch("salt.fileclient.get_file_client")
    @patch("salt.utils.state.get_sls_opts")
    def test_sls(
        self,
        get_sls_opts,
        get_file_client,
        SSHHighState,
        _create_and_execute_salt_state,
    ):
        """
        Test execution of Salt states in chroot.
        """
        SSHHighState.return_value = SSHHighState
        SSHHighState.render_highstate.return_value = (None, [])
        SSHHighState.state.reconcile_extend.return_value = (None, [])
        SSHHighState.state.requisite_in.return_value = (None, [])
        SSHHighState.state.verify_high.return_value = []

        _create_and_execute_salt_state.return_value = "result"
        opts_mock = {
            "hash_type": "md5",
        }
        get_sls_opts.return_value = opts_mock
        with patch.dict(chroot.__opts__, opts_mock):
            self.assertEqual(chroot.sls("/chroot", "module"), "result")
            _create_and_execute_salt_state.assert_called_once()

    @patch("salt.modules.chroot._create_and_execute_salt_state")
    @patch("salt.client.ssh.state.SSHHighState")
    @patch("salt.fileclient.get_file_client")
    @patch("salt.utils.state.get_sls_opts")
    def test_highstate(
        self,
        get_sls_opts,
        get_file_client,
        SSHHighState,
        _create_and_execute_salt_state,
    ):
        """
        Test execution of Salt states in chroot.
        """
        SSHHighState.return_value = SSHHighState

        _create_and_execute_salt_state.return_value = "result"
        opts_mock = {
            "hash_type": "md5",
        }
        get_sls_opts.return_value = opts_mock
        with patch.dict(chroot.__opts__, opts_mock):
            self.assertEqual(chroot.highstate("/chroot"), "result")
            _create_and_execute_salt_state.assert_called_once()



import pytest
import salt.output.highstate as highstate


@pytest.fixture
def configure_loader_modules():
    return {highstate: {"__opts__": {"strip_colors": True}}}


@pytest.mark.parametrize("data", [None, {"return": None}, {"return": {"data": None}}])
def test_when_data_result_is_None_output_should_be_string_None(data):
    expected_output = "None"

    actual_output = highstate.output(data=data)

    assert actual_output == expected_output


def test_when_data_is_dict_with_return_key_and_return_value_has_data_key_and_data_dict_has_one_dict_element_with_jid_and_fun_keys_and_return_value_is_None_then_output_should_return_literal_None_string():

    expected_output = "None"
    data = {
        "return": {
            "data": {
                "foo bar quux fnord": {
                    "jid": "fnordy fnordy fnordy",
                    "fun": "fnordy fnordy fnord",
                    "return": {"data": None},
                },
            }
        },
    }

    actual_output = highstate.output(data=data)

    assert actual_output == expected_output


@pytest.mark.parametrize(
    "return_value",
    [42, "fnord"],
)
def test_when_data_is_dict_with_return_key_and_return_value_has_data_key_and_data_dict_has_one_dict_element_with_jid_and_fun_keys_and_return_value_is_int_or_str_that_value_should_be_returned(
    return_value,
):

    expected_output = return_value
    data = {
        "return": {
            "data": {
                "foo bar quux fnord": {
                    "jid": "fnordy fnordy fnordy",
                    "fun": "fnordy fnordy fnord",
                    "return": {"data": return_value},
                },
            }
        },
    }

    actual_output = highstate.output(data=data)

    assert actual_output == expected_output


def test_when_orchestrator_output_retcode_in_data_the_retcode_should_be_removed():
    data = {"something_master": None, "retcode": 42}

    actual_output = highstate.output(data)

    assert "retcode" not in data


def test_when_more_than_one_local_master_retcode_should_not_be_removed():
    expected_retcode = 42
    data = {
        "something_master": None,
        "another_master": None,
        "retcode": expected_retcode,
    }

    actual_output = highstate.output(data)

    assert data["retcode"] == expected_retcode



import os

import salt.cli.caller
import salt.defaults.exitcodes
import salt.utils.parsers
from salt.config import _expand_glob_path
from salt.utils.verify import verify_log


class SaltCall(salt.utils.parsers.SaltCallOptionParser):
    """
    Used to locally execute a salt command
    """

    def run(self):
        """
        Execute the salt call!
        """
        self.parse_args()

        if self.options.file_root:
            # check if the argument is pointing to a file on disk
            file_root = os.path.abspath(self.options.file_root)
            self.config["file_roots"] = {"base": _expand_glob_path([file_root])}

        if self.options.pillar_root:
            # check if the argument is pointing to a file on disk
            pillar_root = os.path.abspath(self.options.pillar_root)
            self.config["pillar_roots"] = {"base": _expand_glob_path([pillar_root])}

        if self.options.states_dir:
            # check if the argument is pointing to a file on disk
            states_dir = os.path.abspath(self.options.states_dir)
            self.config["states_dirs"] = [states_dir]

        if self.options.local:
            self.config["file_client"] = "local"
        if self.options.master:
            self.config["master"] = self.options.master

        # Setup file logging!
        self.setup_logfile_logger()
        verify_log(self.config)

        caller = salt.cli.caller.Caller.factory(self.config)

        if self.options.doc:
            caller.print_docs()
            self.exit(salt.defaults.exitcodes.EX_OK)

        if self.options.grains_run:
            caller.print_grains()
            self.exit(salt.defaults.exitcodes.EX_OK)

        caller.run()



"""
    :codeauthor: Anthony Shaw <anthonyshaw@apache.org>
"""


import salt.modules.servicenow as servicenow
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock
from tests.support.unit import TestCase


class MockServiceNowClient:
    def __init__(self, instance_name, username, password):
        pass

    def get(self, query):
        return [{"query_size": len(query), "query_value": query}]


class ServiceNowModuleTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        module_globals = {
            "Client": MockServiceNowClient,
            "__salt__": {
                "config.option": MagicMock(
                    return_value={
                        "instance_name": "test",
                        "username": "mr_test",
                        "password": "test123",
                    }
                )
            },
        }
        if servicenow.HAS_LIBS is False:
            module_globals["sys.modules"] = {"servicenow_rest": MagicMock()}
            module_globals["sys.modules"][
                "servicenow_rest"
            ].api.Client = MockServiceNowClient
        return {servicenow: module_globals}

    def test_module_creation(self):
        client = servicenow._get_client()
        self.assertFalse(client is None)

    def test_non_structured_query(self):
        result = servicenow.non_structured_query("tests", "role=web")
        self.assertFalse(result is None)
        self.assertEqual(result[0]["query_size"], 8)
        self.assertEqual(result[0]["query_value"], "role=web")

    def test_non_structured_query_kwarg(self):
        result = servicenow.non_structured_query("tests", role="web")
        self.assertFalse(result is None)
        self.assertEqual(result[0]["query_size"], 8)
        self.assertEqual(result[0]["query_value"], "role=web")

    def test_non_structured_query_kwarg_multi(self):
        result = servicenow.non_structured_query("tests", role="web", type="computer")
        self.assertFalse(result is None)
        self.assertEqual(result[0]["query_size"], 22)



"""
Integration tests for the mac_desktop execution module.
"""

import pytest
from tests.support.case import ModuleCase
from tests.support.helpers import runs_on


@pytest.mark.destructive_test
@runs_on(kernel="Darwin")
@pytest.mark.skip_if_not_root
class MacDesktopTestCase(ModuleCase):
    """
    Integration tests for the mac_desktop module.
    """

    def test_get_output_volume(self):
        """
        Tests the return of get_output_volume.
        """
        ret = self.run_function("desktop.get_output_volume")
        self.assertIsNotNone(ret)

    @pytest.mark.slow_test
    def test_set_output_volume(self):
        """
        Tests the return of set_output_volume.
        """
        current_vol = self.run_function("desktop.get_output_volume")
        to_set = 10
        if current_vol == str(to_set):
            to_set += 2
        new_vol = self.run_function("desktop.set_output_volume", [str(to_set)])
        check_vol = self.run_function("desktop.get_output_volume")
        self.assertEqual(new_vol, check_vol)

        # Set volume back to what it was before
        self.run_function("desktop.set_output_volume", [current_vol])

    def test_screensaver(self):
        """
        Tests the return of the screensaver function.
        """
        self.assertTrue(self.run_function("desktop.screensaver"))

    def test_lock(self):
        """
        Tests the return of the lock function.
        """
        self.assertTrue(self.run_function("desktop.lock"))

    @pytest.mark.slow_test
    def test_say(self):
        """
        Tests the return of the say function.
        """
        self.assertTrue(self.run_function("desktop.say", ["hello", "world"]))



import salt.modules.mac_softwareupdate as mac_softwareupdate
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import patch
from tests.support.unit import TestCase

# These examples include 'recommended' and 'regular' updates. It's
# believed that only 'recommended' updates will ever have an "action", so
# included are examples of 'recommended' updates that have no action, the
# 'restart' action, and the 'shut down' action.

# This may need to be revised as we gather more example output.
MOJAVE_LIST_OUTPUT = """Software Update Tool

Finding available software
Software Update found the following new or updated software:
   * Command Line Tools (macOS Mojave version 10.14) for Xcode-10.3
    Command Line Tools (macOS Mojave version 10.14) for Xcode (10.3), 199140K [recommended]
   * macOS 10.14.1 Update
    macOS 10.14.1 Update (10.14.1), 199140K [recommended] [restart]
   * BridgeOSUpdateCustomer
    BridgeOSUpdateCustomer (10.14.4.1.1.1555388607), 328394K, [recommended] [shut down]
   - iCal-1.0.2
    iCal, (1.0.2), 6520K"""

CATALINA_LIST_OUTPUT = """Software Update Tool

Finding available software
Software Update found the following new or updated software:
* Label: Command Line Tools beta 5 for Xcode-11.0
    Title: Command Line Tools beta 5 for Xcode, Version: 11.0, Size: 224804K, Recommended: YES,
* Label: macOS Catalina Developer Beta-6
    Title: macOS Catalina Public Beta, Version: 5, Size: 3084292K, Recommended: YES, Action: restart,
* Label: BridgeOSUpdateCustomer
    Title: BridgeOSUpdateCustomer, Version: 10.15.0.1.1.1560926689, Size: 390674K, Recommended: YES, Action: shut down,
- Label: iCal-1.0.2
    Title: iCal, Version: 1.0.2, Size: 6520K,"""


class MacSoftwareUpdatePreCatalinaTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        return {mac_softwareupdate: {"__grains__": {"osrelease_info": [10, 14, 6]}}}

    @patch("salt.utils.mac_utils.execute_return_result")
    def test_list_available(self, mock_execute):
        mock_execute.return_value = MOJAVE_LIST_OUTPUT
        result = mac_softwareupdate.list_available()
        expected = {
            "Command Line Tools (macOS Mojave version 10.14) for Xcode-10.3": "10.3",
            "macOS 10.14.1 Update": "10.14.1",
            "BridgeOSUpdateCustomer": "10.14.4.1.1.1555388607",
            "iCal-1.0.2": "1.0.2",
        }
        self.assertEqual(result, expected)

    @patch("salt.utils.mac_utils.execute_return_result")
    def test_list_available_trailing_ws(self, mock_execute):
        """Ensure the regex works with trailing whitespace in labels"""
        # This also tests for no version number returning an empty str.
        # This example's label will not install without the trailing space.
        mock_execute.return_value = (
            "Software Update Tool\n\nFinding available software\nSoftware Update found"
            " the following new or updated software:\n   * macOS Mojave 10.14.6"
            " Supplemental Update- \n    macOS Mojave 10.14.6 Supplemental Update ( ),"
            " 1581834K [recommended] [restart]"
        )
        result = mac_softwareupdate.list_available()
        expected = {"macOS Mojave 10.14.6 Supplemental Update- ": ""}
        self.assertEqual(result, expected)

    @patch("salt.utils.mac_utils.execute_return_result")
    def test_list_recommended(self, mock_execute):
        mock_execute.return_value = MOJAVE_LIST_OUTPUT
        result = mac_softwareupdate.list_available(recommended=True)
        expected = {
            "Command Line Tools (macOS Mojave version 10.14) for Xcode-10.3": "10.3",
            "macOS 10.14.1 Update": "10.14.1",
            "BridgeOSUpdateCustomer": "10.14.4.1.1.1555388607",
        }
        self.assertEqual(result, expected)

    @patch("salt.utils.mac_utils.execute_return_result")
    def test_list_restart(self, mock_execute):
        mock_execute.return_value = MOJAVE_LIST_OUTPUT
        result = mac_softwareupdate.list_available(restart=True)
        expected = {"macOS 10.14.1 Update": "10.14.1"}
        self.assertEqual(result, expected)

    @patch("salt.utils.mac_utils.execute_return_result")
    def test_list_shut_down(self, mock_execute):
        mock_execute.return_value = MOJAVE_LIST_OUTPUT
        result = mac_softwareupdate.list_available(shut_down=True)
        expected = {"BridgeOSUpdateCustomer": "10.14.4.1.1.1555388607"}
        self.assertEqual(result, expected)


class MacSoftwareUpdateCatalinaTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        return {mac_softwareupdate: {"__grains__": {"osrelease_info": [10, 15]}}}

    @patch("salt.utils.mac_utils.execute_return_result")
    def test_list_available(self, mock_execute):
        mock_execute.return_value = CATALINA_LIST_OUTPUT
        result = mac_softwareupdate.list_available()
        expected = {
            "Command Line Tools beta 5 for Xcode-11.0": "11.0",
            "macOS Catalina Developer Beta-6": "5",
            "BridgeOSUpdateCustomer": "10.15.0.1.1.1560926689",
            "iCal-1.0.2": "1.0.2",
        }
        self.assertEqual(result, expected)

    @patch("salt.utils.mac_utils.execute_return_result")
    def test_list_recommended(self, mock_execute):
        mock_execute.return_value = CATALINA_LIST_OUTPUT
        result = mac_softwareupdate.list_available(recommended=True)
        expected = {
            "Command Line Tools beta 5 for Xcode-11.0": "11.0",
            "macOS Catalina Developer Beta-6": "5",
            "BridgeOSUpdateCustomer": "10.15.0.1.1.1560926689",
        }
        self.assertEqual(result, expected)

    @patch("salt.utils.mac_utils.execute_return_result")
    def test_list_restart(self, mock_execute):
        mock_execute.return_value = CATALINA_LIST_OUTPUT
        result = mac_softwareupdate.list_available(restart=True)
        expected = {"macOS Catalina Developer Beta-6": "5"}
        self.assertEqual(result, expected)

    @patch("salt.utils.mac_utils.execute_return_result")
    def test_list_shut_down(self, mock_execute):
        mock_execute.return_value = CATALINA_LIST_OUTPUT
        result = mac_softwareupdate.list_available(shut_down=True)
        expected = {"BridgeOSUpdateCustomer": "10.15.0.1.1.1560926689"}
        self.assertEqual(result, expected)


class MacSoftwareUpdateBigSurTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        return {mac_softwareupdate: {"__grains__": {"osrelease_info": [11, 0]}}}

    @patch("salt.utils.mac_utils.execute_return_result")
    def test_list_available(self, mock_execute):
        mock_execute.return_value = CATALINA_LIST_OUTPUT
        result = mac_softwareupdate.list_available()
        expected = {
            "Command Line Tools beta 5 for Xcode-11.0": "11.0",
            "macOS Catalina Developer Beta-6": "5",
            "BridgeOSUpdateCustomer": "10.15.0.1.1.1560926689",
            "iCal-1.0.2": "1.0.2",
        }
        self.assertEqual(result, expected)

    @patch("salt.utils.mac_utils.execute_return_result")
    def test_list_recommended(self, mock_execute):
        mock_execute.return_value = CATALINA_LIST_OUTPUT
        result = mac_softwareupdate.list_available(recommended=True)
        expected = {
            "Command Line Tools beta 5 for Xcode-11.0": "11.0",
            "macOS Catalina Developer Beta-6": "5",
            "BridgeOSUpdateCustomer": "10.15.0.1.1.1560926689",
        }
        self.assertEqual(result, expected)

    @patch("salt.utils.mac_utils.execute_return_result")
    def test_list_restart(self, mock_execute):
        mock_execute.return_value = CATALINA_LIST_OUTPUT
        result = mac_softwareupdate.list_available(restart=True)
        expected = {"macOS Catalina Developer Beta-6": "5"}
        self.assertEqual(result, expected)

    @patch("salt.utils.mac_utils.execute_return_result")
    def test_list_shut_down(self, mock_execute):
        mock_execute.return_value = CATALINA_LIST_OUTPUT
        result = mac_softwareupdate.list_available(shut_down=True)
        expected = {"BridgeOSUpdateCustomer": "10.15.0.1.1.1560926689"}
        self.assertEqual(result, expected)



import pytest
import salt.states.macdefaults as macdefaults
from tests.support.mock import MagicMock, patch


@pytest.fixture
def configure_loader_modules():
    return {macdefaults: {}}


def test_write():
    """
    Test writing a default setting
    """
    expected = {
        "changes": {"written": "com.apple.CrashReporter DialogType is set to Server"},
        "comment": "",
        "name": "DialogType",
        "result": True,
    }

    read_mock = MagicMock(return_value="Local")
    write_mock = MagicMock(return_value={"retcode": 0})
    with patch.dict(
        macdefaults.__salt__,
        {"macdefaults.read": read_mock, "macdefaults.write": write_mock},
    ):
        out = macdefaults.write("DialogType", "com.apple.CrashReporter", "Server")
        read_mock.assert_called_once_with("com.apple.CrashReporter", "DialogType", None)
        write_mock.assert_called_once_with(
            "com.apple.CrashReporter", "DialogType", "Server", "string", None
        )
        assert out == expected


def test_write_set():
    """
    Test writing a default setting that is already set
    """
    expected = {
        "changes": {},
        "comment": "com.apple.CrashReporter DialogType is already set to Server",
        "name": "DialogType",
        "result": True,
    }

    read_mock = MagicMock(return_value="Server")
    write_mock = MagicMock(return_value={"retcode": 0})
    with patch.dict(
        macdefaults.__salt__,
        {"macdefaults.read": read_mock, "macdefaults.write": write_mock},
    ):
        out = macdefaults.write("DialogType", "com.apple.CrashReporter", "Server")
        read_mock.assert_called_once_with("com.apple.CrashReporter", "DialogType", None)
        assert not write_mock.called
        assert out == expected


def test_write_boolean():
    """
    Test writing a default setting with a boolean
    """
    expected = {
        "changes": {"written": "com.apple.something Key is set to True"},
        "comment": "",
        "name": "Key",
        "result": True,
    }

    read_mock = MagicMock(return_value="0")
    write_mock = MagicMock(return_value={"retcode": 0})
    with patch.dict(
        macdefaults.__salt__,
        {"macdefaults.read": read_mock, "macdefaults.write": write_mock},
    ):
        out = macdefaults.write("Key", "com.apple.something", True, vtype="boolean")
        read_mock.assert_called_once_with("com.apple.something", "Key", None)
        write_mock.assert_called_once_with(
            "com.apple.something", "Key", True, "boolean", None
        )
        assert out == expected


def test_write_boolean_match():
    """
    Test writing a default setting with a boolean that is already set to the same value
    """
    expected = {
        "changes": {},
        "comment": "com.apple.something Key is already set to YES",
        "name": "Key",
        "result": True,
    }

    read_mock = MagicMock(return_value="1")
    write_mock = MagicMock(return_value={"retcode": 0})
    with patch.dict(
        macdefaults.__salt__,
        {"macdefaults.read": read_mock, "macdefaults.write": write_mock},
    ):
        out = macdefaults.write("Key", "com.apple.something", "YES", vtype="boolean")
        read_mock.assert_called_once_with("com.apple.something", "Key", None)
        assert not write_mock.called
        assert out == expected


def test_write_integer():
    """
    Test writing a default setting with a integer
    """
    expected = {
        "changes": {"written": "com.apple.something Key is set to 1337"},
        "comment": "",
        "name": "Key",
        "result": True,
    }

    read_mock = MagicMock(return_value="99")
    write_mock = MagicMock(return_value={"retcode": 0})
    with patch.dict(
        macdefaults.__salt__,
        {"macdefaults.read": read_mock, "macdefaults.write": write_mock},
    ):
        out = macdefaults.write("Key", "com.apple.something", 1337, vtype="integer")
        read_mock.assert_called_once_with("com.apple.something", "Key", None)
        write_mock.assert_called_once_with(
            "com.apple.something", "Key", 1337, "integer", None
        )
        assert out == expected


def test_write_integer_match():
    """
    Test writing a default setting with a integer that is already set to the same value
    """
    expected = {
        "changes": {},
        "comment": "com.apple.something Key is already set to 1337",
        "name": "Key",
        "result": True,
    }

    read_mock = MagicMock(return_value="1337")
    write_mock = MagicMock(return_value={"retcode": 0})
    with patch.dict(
        macdefaults.__salt__,
        {"macdefaults.read": read_mock, "macdefaults.write": write_mock},
    ):
        out = macdefaults.write("Key", "com.apple.something", 1337, vtype="integer")
        read_mock.assert_called_once_with("com.apple.something", "Key", None)
        assert not write_mock.called
        assert out == expected


def test_absent_already():
    """
    Test ensuring non-existent defaults value is absent
    """
    expected = {
        "changes": {},
        "comment": "com.apple.something Key is already absent",
        "name": "Key",
        "result": True,
    }

    mock = MagicMock(return_value={"retcode": 1})
    with patch.dict(macdefaults.__salt__, {"macdefaults.delete": mock}):
        out = macdefaults.absent("Key", "com.apple.something")
        mock.assert_called_once_with("com.apple.something", "Key", None)
        assert out == expected


def test_absent_deleting_existing():
    """
    Test removing an existing value
    """
    expected = {
        "changes": {"absent": "com.apple.something Key is now absent"},
        "comment": "",
        "name": "Key",
        "result": True,
    }

    mock = MagicMock(return_value={"retcode": 0})
    with patch.dict(macdefaults.__salt__, {"macdefaults.delete": mock}):
        out = macdefaults.absent("Key", "com.apple.something")
        mock.assert_called_once_with("com.apple.something", "Key", None)
        assert out == expected



"""
Unit tests for the Snapper module

:codeauthor:    Duncan Mac-Vicar P. <dmacvicar@suse.de>
:codeauthor:    Pablo Suárez Hernández <psuarezhernandez@suse.de>
"""


import sys

import salt.modules.snapper as snapper
import salt.utils.files
import salt.utils.platform
from salt.exceptions import CommandExecutionError
from tests.support.helpers import with_tempfile
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, mock_open, patch
from tests.support.unit import TestCase, skipIf

DBUS_RET = {
    "ListSnapshots": [
        [
            42,
            1,
            0,
            1457006571,
            0,
            "Some description",
            "",
            {"userdata1": "userval1", "salt_jid": "20160607130930720112"},
        ],
        [
            43,
            2,
            42,
            1457006572,
            0,
            "Blah Blah",
            "",
            {"userdata2": "userval2", "salt_jid": "20160607130930720112"},
        ],
    ],
    "ListConfigs": [
        [
            "root",
            "/",
            {
                "SUBVOLUME": "/",
                "NUMBER_MIN_AGE": "1800",
                "TIMELINE_LIMIT_YEARLY": "4-10",
                "NUMBER_LIMIT_IMPORTANT": "10",
                "FSTYPE": "btrfs",
                "TIMELINE_LIMIT_MONTHLY": "4-10",
                "ALLOW_GROUPS": "",
                "EMPTY_PRE_POST_MIN_AGE": "1800",
                "EMPTY_PRE_POST_CLEANUP": "yes",
                "BACKGROUND_COMPARISON": "yes",
                "TIMELINE_LIMIT_HOURLY": "4-10",
                "ALLOW_USERS": "",
                "TIMELINE_LIMIT_WEEKLY": "0",
                "TIMELINE_CREATE": "no",
                "NUMBER_CLEANUP": "yes",
                "TIMELINE_CLEANUP": "yes",
                "SPACE_LIMIT": "0.5",
                "NUMBER_LIMIT": "10",
                "TIMELINE_MIN_AGE": "1800",
                "TIMELINE_LIMIT_DAILY": "4-10",
                "SYNC_ACL": "no",
                "QGROUP": "1/0",
            },
        ]
    ],
    "GetFiles": [
        ["/root/.viminfo", 8],
        ["/tmp/foo", 52],
        ["/tmp/foo2", 1],
        ["/tmp/foo3", 2],
        ["/var/log/snapper.log", 8],
        ["/var/cache/salt/minion/extmods/modules/snapper.py", 8],
        ["/var/cache/salt/minion/extmods/modules/snapper.pyc", 8],
    ],
}

FILE_CONTENT = {
    "/tmp/foo": {"pre": "dummy text", "post": "another foobar"},
    "/tmp/foo2": {"post": "another foobar"},
}

MODULE_RET = {
    "SNAPSHOTS": [
        {
            "userdata": {"userdata1": "userval1", "salt_jid": "20160607130930720112"},
            "description": "Some description",
            "timestamp": 1457006571,
            "cleanup": "",
            "user": "root",
            "type": "pre",
            "id": 42,
        },
        {
            "pre": 42,
            "userdata": {"userdata2": "userval2", "salt_jid": "20160607130930720112"},
            "description": "Blah Blah",
            "timestamp": 1457006572,
            "cleanup": "",
            "user": "root",
            "type": "post",
            "id": 43,
        },
    ],
    "LISTCONFIGS": {
        "root": {
            "SUBVOLUME": "/",
            "NUMBER_MIN_AGE": "1800",
            "TIMELINE_LIMIT_YEARLY": "4-10",
            "NUMBER_LIMIT_IMPORTANT": "10",
            "FSTYPE": "btrfs",
            "TIMELINE_LIMIT_MONTHLY": "4-10",
            "ALLOW_GROUPS": "",
            "EMPTY_PRE_POST_MIN_AGE": "1800",
            "EMPTY_PRE_POST_CLEANUP": "yes",
            "BACKGROUND_COMPARISON": "yes",
            "TIMELINE_LIMIT_HOURLY": "4-10",
            "ALLOW_USERS": "",
            "TIMELINE_LIMIT_WEEKLY": "0",
            "TIMELINE_CREATE": "no",
            "NUMBER_CLEANUP": "yes",
            "TIMELINE_CLEANUP": "yes",
            "SPACE_LIMIT": "0.5",
            "NUMBER_LIMIT": "10",
            "TIMELINE_MIN_AGE": "1800",
            "TIMELINE_LIMIT_DAILY": "4-10",
            "SYNC_ACL": "no",
            "QGROUP": "1/0",
        }
    },
    "GETFILES": {
        "/root/.viminfo": {"status": ["modified"]},
        "/tmp/foo": {"status": ["type changed", "permission changed", "owner changed"]},
        "/tmp/foo2": {"status": ["created"]},
        "/tmp/foo3": {"status": ["deleted"]},
        "/var/log/snapper.log": {"status": ["modified"]},
        "/var/cache/salt/minion/extmods/modules/snapper.py": {"status": ["modified"]},
        "/var/cache/salt/minion/extmods/modules/snapper.pyc": {"status": ["modified"]},
    },
    "DIFF": {
        "/tmp/foo": {
            "comment": "text file changed",
            "diff": (
                "--- /.snapshots/55/snapshot/tmp/foo\n"
                "+++ /tmp/foo\n"
                "@@ -1 +1 @@\n"
                "-dummy text"
                "+another foobar"
            ),
        },
        "/tmp/foo2": {
            "comment": "text file created",
            "diff": (
                "--- /.snapshots/55/snapshot/tmp/foo2\n"
                "+++ /tmp/foo2\n"
                "@@ -0,0 +1 @@\n"
                "+another foobar"
            ),
        },
        "/tmp/foo26": {
            "comment": "text file created",
            "diff": (
                "--- /.snapshots/55/snapshot/tmp/foo2 \n"
                "+++ /tmp/foo2 \n"
                "@@ -1,0 +1,1 @@\n"
                "+another foobar"
            ),
        },
        "/tmp/foo3": {
            "comment": "binary file changed",
            "old_sha256_digest": (
                "e61f8b762d83f3b4aeb3689564b0ffbe54fa731a69a1e208dc9440ce0f69d19b"
            ),
            "new_sha256_digest": (
                "f18f971f1517449208a66589085ddd3723f7f6cefb56c141e3d97ae49e1d87fa"
            ),
        },
    },
}


@skipIf(sys.platform.startswith("win"), "Snapper not available on Windows")
class SnapperTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        class DBusException(BaseException):
            get_dbus_name = "foo"

        dbus_mock = MagicMock()
        dbus_mock.configure_mock(DBusException=DBusException)
        return {snapper: {"dbus": dbus_mock, "snapper": MagicMock()}}

    def test__snapshot_to_data(self):
        data = snapper._snapshot_to_data(
            DBUS_RET["ListSnapshots"][0]
        )  # pylint: disable=protected-access
        self.assertEqual(data["id"], 42)
        self.assertNotIn("pre", data)
        self.assertEqual(data["type"], "pre")
        self.assertEqual(data["user"], "root")
        self.assertEqual(data["timestamp"], 1457006571)
        self.assertEqual(data["description"], "Some description")
        self.assertEqual(data["cleanup"], "")
        self.assertEqual(data["userdata"]["userdata1"], "userval1")

    def test_list_snapshots(self):
        with patch(
            "salt.modules.snapper.snapper.ListSnapshots",
            MagicMock(return_value=DBUS_RET["ListSnapshots"]),
        ):
            self.assertEqual(snapper.list_snapshots(), MODULE_RET["SNAPSHOTS"])

    def test_get_snapshot(self):
        with patch(
            "salt.modules.snapper.snapper.GetSnapshot",
            MagicMock(return_value=DBUS_RET["ListSnapshots"][0]),
        ):
            self.assertEqual(snapper.get_snapshot(), MODULE_RET["SNAPSHOTS"][0])
            self.assertEqual(
                snapper.get_snapshot(number=42), MODULE_RET["SNAPSHOTS"][0]
            )
            self.assertNotEqual(
                snapper.get_snapshot(number=42), MODULE_RET["SNAPSHOTS"][1]
            )

    def test_list_configs(self):
        with patch(
            "salt.modules.snapper.snapper.ListConfigs",
            MagicMock(return_value=DBUS_RET["ListConfigs"]),
        ):
            self.assertEqual(snapper.list_configs(), MODULE_RET["LISTCONFIGS"])

    def test_get_config(self):
        with patch(
            "salt.modules.snapper.snapper.GetConfig",
            MagicMock(return_value=DBUS_RET["ListConfigs"][0]),
        ):
            self.assertEqual(snapper.get_config(), DBUS_RET["ListConfigs"][0])

    def test_set_config(self):
        with patch("salt.modules.snapper.snapper.SetConfig", MagicMock()):
            opts = {"sync_acl": True, "dummy": False, "foobar": 1234}
            self.assertEqual(snapper.set_config(opts), True)

    def test_status_to_string(self):
        self.assertEqual(snapper.status_to_string(1), ["created"])
        self.assertEqual(snapper.status_to_string(2), ["deleted"])
        self.assertEqual(snapper.status_to_string(4), ["type changed"])
        self.assertEqual(snapper.status_to_string(8), ["modified"])
        self.assertEqual(snapper.status_to_string(16), ["permission changed"])
        self.assertListEqual(
            snapper.status_to_string(24), ["modified", "permission changed"]
        )
        self.assertEqual(snapper.status_to_string(32), ["owner changed"])
        self.assertEqual(snapper.status_to_string(64), ["group changed"])
        self.assertListEqual(
            snapper.status_to_string(97), ["created", "owner changed", "group changed"]
        )
        self.assertEqual(snapper.status_to_string(128), ["extended attributes changed"])
        self.assertEqual(snapper.status_to_string(256), ["ACL info changed"])

    def test_create_config(self):
        with patch("salt.modules.snapper.snapper.CreateConfig", MagicMock()), patch(
            "salt.modules.snapper.snapper.GetConfig",
            MagicMock(return_value=DBUS_RET["ListConfigs"][0]),
        ):
            opts = {
                "name": "testconfig",
                "subvolume": "/foo/bar/",
                "fstype": "btrfs",
                "template": "mytemplate",
                "extra_opts": {"NUMBER_CLEANUP": False},
            }
            with patch(
                "salt.modules.snapper.set_config", MagicMock()
            ) as set_config_mock:
                self.assertEqual(
                    snapper.create_config(**opts), DBUS_RET["ListConfigs"][0]
                )
                set_config_mock.assert_called_with("testconfig", **opts["extra_opts"])

            with patch(
                "salt.modules.snapper.set_config", MagicMock()
            ) as set_config_mock:
                del opts["extra_opts"]
                self.assertEqual(
                    snapper.create_config(**opts), DBUS_RET["ListConfigs"][0]
                )
                assert not set_config_mock.called
                self.assertRaises(CommandExecutionError, snapper.create_config)

    def test_create_snapshot(self):
        with patch(
            "salt.modules.snapper.snapper.CreateSingleSnapshot",
            MagicMock(return_value=1234),
        ), patch(
            "salt.modules.snapper.snapper.CreatePreSnapshot",
            MagicMock(return_value=1234),
        ), patch(
            "salt.modules.snapper.snapper.CreatePostSnapshot",
            MagicMock(return_value=1234),
        ):
            for snapshot_type in ["pre", "post", "single"]:
                opts = {
                    "__pub_jid": 20160607130930720112,
                    "type": snapshot_type,
                    "description": "Test description",
                    "cleanup_algorithm": "number",
                    "pre_number": 23,
                }
                self.assertEqual(snapper.create_snapshot(**opts), 1234)

    def test_delete_snapshot_id_success(self):
        with patch("salt.modules.snapper.snapper.DeleteSnapshots", MagicMock()), patch(
            "salt.modules.snapper.snapper.ListSnapshots",
            MagicMock(return_value=DBUS_RET["ListSnapshots"]),
        ):
            self.assertEqual(
                snapper.delete_snapshot(snapshots_ids=43),
                {"root": {"ids": [43], "status": "deleted"}},
            )
            self.assertEqual(
                snapper.delete_snapshot(snapshots_ids=[42, 43]),
                {"root": {"ids": [42, 43], "status": "deleted"}},
            )

    def test_delete_snapshot_id_fail(self):
        with patch("salt.modules.snapper.snapper.DeleteSnapshots", MagicMock()), patch(
            "salt.modules.snapper.snapper.ListSnapshots",
            MagicMock(return_value=DBUS_RET["ListSnapshots"]),
        ):
            self.assertRaises(CommandExecutionError, snapper.delete_snapshot)
            self.assertRaises(
                CommandExecutionError, snapper.delete_snapshot, snapshots_ids=1
            )
            self.assertRaises(
                CommandExecutionError, snapper.delete_snapshot, snapshots_ids=[1, 2]
            )

    def test_modify_snapshot(self):
        with patch("salt.modules.snapper.snapper.SetSnapshot", MagicMock()):
            _ret = {
                "userdata": {"userdata2": "uservalue2"},
                "description": "UPDATED DESCRIPTION",
                "timestamp": 1457006571,
                "cleanup": "number",
                "user": "root",
                "type": "pre",
                "id": 42,
            }
            _opts = {
                "config": "root",
                "snapshot_id": 42,
                "cleanup": "number",
                "description": "UPDATED DESCRIPTION",
                "userdata": {"userdata2": "uservalue2"},
            }
            with patch(
                "salt.modules.snapper.get_snapshot",
                MagicMock(side_effect=[DBUS_RET["ListSnapshots"][0], _ret]),
            ):
                self.assertDictEqual(snapper.modify_snapshot(**_opts), _ret)

    def test__get_num_interval(self):
        with patch(
            "salt.modules.snapper._get_last_snapshot",
            MagicMock(return_value={"id": 42}),
        ):
            self.assertEqual(
                snapper._get_num_interval(config=None, num_pre=None, num_post=None),
                (42, 0),
            )  # pylint: disable=protected-access
            self.assertEqual(
                snapper._get_num_interval(config=None, num_pre=None, num_post=50),
                (42, 50),
            )  # pylint: disable=protected-access
            self.assertEqual(
                snapper._get_num_interval(config=None, num_pre=42, num_post=50),
                (42, 50),
            )  # pylint: disable=protected-access

    def test_run(self):
        patch_dict = {
            "snapper.create_snapshot": MagicMock(return_value=43),
            "test.ping": MagicMock(return_value=True),
        }
        with patch.dict(snapper.__salt__, patch_dict):
            self.assertEqual(snapper.run("test.ping"), True)
            self.assertRaises(CommandExecutionError, snapper.run, "unknown.func")

    def test_status(self):
        with patch(
            "salt.modules.snapper._get_num_interval", MagicMock(return_value=(42, 43))
        ), patch("salt.modules.snapper.snapper.GetComparison", MagicMock()), patch(
            "salt.modules.snapper.snapper.GetFiles",
            MagicMock(return_value=DBUS_RET["GetFiles"]),
        ), patch(
            "salt.modules.snapper.snapper.ListConfigs",
            MagicMock(return_value=DBUS_RET["ListConfigs"]),
        ):
            self.assertCountEqual(snapper.status(), MODULE_RET["GETFILES"])
            self.assertCountEqual(
                snapper.status(num_pre="42", num_post=43), MODULE_RET["GETFILES"]
            )
            self.assertCountEqual(snapper.status(num_pre=42), MODULE_RET["GETFILES"])
            self.assertCountEqual(snapper.status(num_post=43), MODULE_RET["GETFILES"])

    def test_changed_files(self):
        with patch(
            "salt.modules.snapper.status",
            MagicMock(return_value=MODULE_RET["GETFILES"]),
        ):
            self.assertEqual(snapper.changed_files(), MODULE_RET["GETFILES"].keys())

    def test_undo(self):
        with patch(
            "salt.modules.snapper._get_num_interval", MagicMock(return_value=(42, 43))
        ), patch(
            "salt.modules.snapper.status",
            MagicMock(return_value=MODULE_RET["GETFILES"]),
        ):
            cmd_ret = "create:0 modify:1 delete:0"
            with patch.dict(
                snapper.__salt__, {"cmd.run": MagicMock(return_value=cmd_ret)}
            ):
                module_ret = {"create": "0", "delete": "0", "modify": "1"}
                self.assertEqual(snapper.undo(files=["/tmp/foo"]), module_ret)

            cmd_ret = "create:1 modify:1 delete:0"
            with patch.dict(
                snapper.__salt__, {"cmd.run": MagicMock(return_value=cmd_ret)}
            ):
                module_ret = {"create": "1", "delete": "0", "modify": "1"}
                self.assertEqual(
                    snapper.undo(files=["/tmp/foo", "/tmp/foo2"]), module_ret
                )

            cmd_ret = "create:1 modify:1 delete:1"
            with patch.dict(
                snapper.__salt__, {"cmd.run": MagicMock(return_value=cmd_ret)}
            ):
                module_ret = {"create": "1", "delete": "1", "modify": "1"}
                self.assertEqual(
                    snapper.undo(files=["/tmp/foo", "/tmp/foo2", "/tmp/foo3"]),
                    module_ret,
                )

    def test__get_jid_snapshots(self):
        with patch(
            "salt.modules.snapper.list_snapshots",
            MagicMock(return_value=MODULE_RET["SNAPSHOTS"]),
        ):
            self.assertEqual(
                snapper._get_jid_snapshots(
                    "20160607130930720112"
                ),  # pylint: disable=protected-access
                (MODULE_RET["SNAPSHOTS"][0]["id"], MODULE_RET["SNAPSHOTS"][1]["id"]),
            )

    def test_undo_jid(self):
        with patch(
            "salt.modules.snapper._get_jid_snapshots", MagicMock(return_value=(42, 43))
        ), patch(
            "salt.modules.snapper.undo",
            MagicMock(return_value="create:1 modify:1 delete:1"),
        ):
            self.assertEqual(
                snapper.undo_jid(20160607130930720112), "create:1 modify:1 delete:1"
            )

    def test_diff_text_file(self):
        with patch(
            "salt.modules.snapper._get_num_interval", MagicMock(return_value=(42, 43))
        ), patch(
            "salt.modules.snapper.snapper.MountSnapshot",
            MagicMock(side_effect=["/.snapshots/55/snapshot", ""]),
        ), patch(
            "salt.modules.snapper.snapper.UmountSnapshot", MagicMock(return_value="")
        ), patch(
            "os.path.isdir", MagicMock(return_value=False)
        ), patch(
            "salt.modules.snapper.changed_files", MagicMock(return_value=["/tmp/foo2"])
        ), patch(
            "salt.modules.snapper._is_text_file", MagicMock(return_value=True)
        ), patch(
            "os.path.isfile", MagicMock(side_effect=[False, True])
        ), patch(
            "salt.utils.files.fopen",
            mock_open(read_data=FILE_CONTENT["/tmp/foo2"]["post"]),
        ), patch(
            "salt.modules.snapper.snapper.ListConfigs",
            MagicMock(return_value=DBUS_RET["ListConfigs"]),
        ):
            if sys.version_info < (2, 7):
                self.assertEqual(
                    snapper.diff(), {"/tmp/foo2": MODULE_RET["DIFF"]["/tmp/foo26"]}
                )
            else:
                self.assertEqual(
                    snapper.diff(), {"/tmp/foo2": MODULE_RET["DIFF"]["/tmp/foo2"]}
                )

    @skipIf(sys.version_info < (2, 7), "Python 2.7 required to compare diff properly")
    def test_diff_text_files(self):
        with patch(
            "salt.modules.snapper._get_num_interval", MagicMock(return_value=(55, 0))
        ), patch(
            "salt.modules.snapper.snapper.MountSnapshot",
            MagicMock(
                side_effect=[
                    "/.snapshots/55/snapshot",
                    "",
                    "/.snapshots/55/snapshot",
                    "",
                ]
            ),
        ), patch(
            "salt.modules.snapper.snapper.UmountSnapshot", MagicMock(return_value="")
        ), patch(
            "salt.modules.snapper.changed_files",
            MagicMock(return_value=["/tmp/foo", "/tmp/foo2"]),
        ), patch(
            "salt.modules.snapper._is_text_file", MagicMock(return_value=True)
        ), patch(
            "os.path.isfile", MagicMock(side_effect=[True, True, False, True])
        ), patch(
            "os.path.isdir", MagicMock(return_value=False)
        ), patch(
            "salt.modules.snapper.snapper.ListConfigs",
            MagicMock(return_value=DBUS_RET["ListConfigs"]),
        ):
            contents = {
                "*/tmp/foo": [
                    FILE_CONTENT["/tmp/foo"]["pre"],
                    FILE_CONTENT["/tmp/foo"]["post"],
                ],
                "*/tmp/foo2": FILE_CONTENT["/tmp/foo2"]["post"],
            }
            with patch("salt.utils.files.fopen", mock_open(read_data=contents)):
                module_ret = {
                    "/tmp/foo": MODULE_RET["DIFF"]["/tmp/foo"],
                    "/tmp/foo2": MODULE_RET["DIFF"]["/tmp/foo2"],
                }
                self.assertEqual(snapper.diff(), module_ret)

    def test_diff_binary_files(self):
        with patch(
            "salt.modules.snapper._get_num_interval", MagicMock(return_value=(55, 0))
        ), patch(
            "salt.modules.snapper.snapper.MountSnapshot",
            MagicMock(
                side_effect=[
                    "/.snapshots/55/snapshot",
                    "",
                    "/.snapshots/55/snapshot",
                    "",
                ]
            ),
        ), patch(
            "salt.modules.snapper.snapper.UmountSnapshot", MagicMock(return_value="")
        ), patch(
            "salt.modules.snapper.changed_files", MagicMock(return_value=["/tmp/foo3"])
        ), patch(
            "salt.modules.snapper._is_text_file", MagicMock(return_value=False)
        ), patch(
            "os.path.isfile", MagicMock(side_effect=[True, True])
        ), patch(
            "os.path.isdir", MagicMock(return_value=False)
        ), patch(
            "salt.modules.snapper.snapper.ListConfigs",
            MagicMock(return_value=DBUS_RET["ListConfigs"]),
        ), patch.dict(
            snapper.__salt__,
            {
                "hashutil.sha256_digest": MagicMock(
                    side_effect=[
                        "e61f8b762d83f3b4aeb3689564b0ffbe54fa731a69a1e208dc9440ce0f69d19b",
                        "f18f971f1517449208a66589085ddd3723f7f6cefb56c141e3d97ae49e1d87fa",
                    ]
                )
            },
        ):
            with patch("salt.utils.files.fopen", mock_open(read_data="dummy binary")):
                module_ret = {
                    "/tmp/foo3": MODULE_RET["DIFF"]["/tmp/foo3"],
                }
                self.assertEqual(snapper.diff(), module_ret)

    @skipIf(salt.utils.platform.is_linux() is False, "This is a linux only test")
    @with_tempfile()
    def test__is_text_file(self, tempfile):
        with salt.utils.files.fopen(tempfile, "w") as wfh:
            wfh.write(
                "Once upon a time there was an old Sow with three little Pigs, and "
                "as she had not enough to keep them, she sent them out to seek their "
                "fortune.\n"
            )
        assert snapper._is_text_file(tempfile) is True

        with salt.utils.files.fopen(tempfile, "wb") as wfh:
            wfh.write(
                b"C\x07\xd6\x13\xe5_\x99D\xeb\xd7v\xc1\x96p\x84\xd2{a\x03++\r\xcd/"
                b"\xdb\x98\xda\xf7H\xf8\xfb-\x95\xa9}|^\t\xddx\x1c\x18s\x1bZ\x86\x8a(S\xe4"
            )
        assert snapper._is_text_file(tempfile) is False



"""
Common functions for managing package refreshes during states
"""

import errno
import logging
import os
import re

import salt.utils.data
import salt.utils.files
import salt.utils.versions

log = logging.getLogger(__name__)


def rtag(opts):
    """
    Return the rtag file location. This file is used to ensure that we don't
    refresh more than once (unless explicitly configured to do so).
    """
    return os.path.join(opts["cachedir"], "pkg_refresh")


def clear_rtag(opts):
    """
    Remove the rtag file
    """
    try:
        os.remove(rtag(opts))
    except OSError as exc:
        if exc.errno != errno.ENOENT:
            # Using __str__() here to get the fully-formatted error message
            # (error number, error message, path)
            log.warning("Encountered error removing rtag: %s", exc.__str__())


def write_rtag(opts):
    """
    Write the rtag file
    """
    rtag_file = rtag(opts)
    if not os.path.exists(rtag_file):
        try:
            with salt.utils.files.fopen(rtag_file, "w+"):
                pass
        except OSError as exc:
            log.warning("Encountered error writing rtag: %s", exc.__str__())


def check_refresh(opts, refresh=None):
    """
    Check whether or not a refresh is necessary

    Returns:

    - True if refresh evaluates as True
    - False if refresh is False
    - A boolean if refresh is not False and the rtag file exists
    """
    return bool(
        salt.utils.data.is_true(refresh)
        or (os.path.isfile(rtag(opts)) and refresh is not False)
    )


def split_comparison(version):
    match = re.match(r"^(<=>|!=|>=|<=|>>|<<|<>|>|<|=)?\s?([^<>=]+)$", version)
    if match:
        comparison = match.group(1) or ""
        version = match.group(2)
    else:
        comparison = ""
    return comparison, version


def match_version(desired, available, cmp_func=None, ignore_epoch=False):
    """
    Returns the first version of the list of available versions which matches
    the desired version comparison expression, or None if no match is found.
    """
    oper, version = split_comparison(desired)
    if not oper:
        oper = "=="
    for candidate in available:
        if salt.utils.versions.compare(
            ver1=candidate,
            oper=oper,
            ver2=version,
            cmp_func=cmp_func,
            ignore_epoch=ignore_epoch,
        ):
            return candidate
    return None



"""
Management of SELinux rules
===========================

If SELinux is available for the running system, the mode can be managed and
booleans can be set.

.. code-block:: yaml

    enforcing:
        selinux.mode

    samba_create_home_dirs:
        selinux.boolean:
          - value: True
          - persist: True

    nginx:
        selinux.module:
          - enabled: False

.. note::
    Use of these states require that the :mod:`selinux <salt.modules.selinux>`
    execution module is available.
"""


def __virtual__():
    """
    Only make this state available if the selinux module is available.
    """
    if "selinux.getenforce" in __salt__:
        return "selinux"
    return (False, "selinux module could not be loaded")


def _refine_mode(mode):
    """
    Return a mode value that is predictable
    """
    mode = str(mode).lower()
    if any([mode.startswith("e"), mode == "1", mode == "on"]):
        return "Enforcing"
    if any([mode.startswith("p"), mode == "0", mode == "off"]):
        return "Permissive"
    if any([mode.startswith("d")]):
        return "Disabled"
    return "unknown"


def _refine_value(value):
    """
    Return a yes/no value, or None if the input is invalid
    """
    value = str(value).lower()
    if value in ("1", "on", "yes", "true"):
        return "on"
    if value in ("0", "off", "no", "false"):
        return "off"
    return None


def _refine_module_state(module_state):
    """
    Return a predictable value, or allow us to error out
    .. versionadded:: 2016.3.0
    """
    module_state = str(module_state).lower()
    if module_state in ("1", "on", "yes", "true", "enabled"):
        return "enabled"
    if module_state in ("0", "off", "no", "false", "disabled"):
        return "disabled"
    return "unknown"


def mode(name):
    """
    Verifies the mode SELinux is running in, can be set to enforcing,
    permissive, or disabled

    .. note::
        A change to or from disabled mode requires a system reboot. You will
        need to perform this yourself.

    name
        The mode to run SELinux in, permissive, enforcing, or disabled.
    """
    ret = {"name": name, "result": False, "comment": "", "changes": {}}
    tmode = _refine_mode(name)
    if tmode == "unknown":
        ret["comment"] = "{} is not an accepted mode".format(name)
        return ret
    # Either the current mode in memory or a non-matching config value
    # will trigger setenforce
    mode = __salt__["selinux.getenforce"]()
    config = __salt__["selinux.getconfig"]()
    # Just making sure the oldmode reflects the thing that didn't match tmode
    if mode == tmode and mode != config and tmode != config:
        mode = config

    if mode == tmode:
        ret["result"] = True
        ret["comment"] = "SELinux is already in {} mode".format(tmode)
        return ret
    # The mode needs to change...
    if __opts__["test"]:
        ret["comment"] = "SELinux mode is set to be changed to {}".format(tmode)
        ret["result"] = None
        ret["changes"] = {"old": mode, "new": tmode}
        return ret

    oldmode, mode = mode, __salt__["selinux.setenforce"](tmode)
    if mode == tmode or (
        tmode == "Disabled" and __salt__["selinux.getconfig"]() == tmode
    ):
        ret["result"] = True
        ret["comment"] = "SELinux has been set to {} mode".format(tmode)
        ret["changes"] = {"old": oldmode, "new": mode}
        return ret
    ret["comment"] = "Failed to set SELinux to {} mode".format(tmode)
    return ret


def boolean(name, value, persist=False):
    """
    Set up an SELinux boolean

    name
        The name of the boolean to set

    value
        The value to set on the boolean

    persist
        Defaults to False, set persist to true to make the boolean apply on a
        reboot
    """
    ret = {"name": name, "result": True, "comment": "", "changes": {}}
    bools = __salt__["selinux.list_sebool"]()
    if name not in bools:
        ret["comment"] = "Boolean {} is not available".format(name)
        ret["result"] = False
        return ret
    rvalue = _refine_value(value)
    if rvalue is None:
        ret["comment"] = "{} is not a valid value for the boolean".format(value)
        ret["result"] = False
        return ret
    state = bools[name]["State"] == rvalue
    default = bools[name]["Default"] == rvalue
    if persist:
        if state and default:
            ret["comment"] = "Boolean is in the correct state"
            return ret
    else:
        if state:
            ret["comment"] = "Boolean is in the correct state"
            return ret
    if __opts__["test"]:
        ret["result"] = None
        ret["comment"] = "Boolean {} is set to be changed to {}".format(name, rvalue)
        return ret

    ret["result"] = __salt__["selinux.setsebool"](name, rvalue, persist)
    if ret["result"]:
        ret["comment"] = "Boolean {} has been set to {}".format(name, rvalue)
        ret["changes"].update({"State": {"old": bools[name]["State"], "new": rvalue}})
        if persist and not default:
            ret["changes"].update(
                {"Default": {"old": bools[name]["Default"], "new": rvalue}}
            )
        return ret
    ret["comment"] = "Failed to set the boolean {} to {}".format(name, rvalue)
    return ret


def module(name, module_state="Enabled", version="any", **opts):
    """
    Enable/Disable and optionally force a specific version for an SELinux module

    name
        The name of the module to control

    module_state
        Should the module be enabled or disabled?

    version
        Defaults to no preference, set to a specified value if required.
        Currently can only alert if the version is incorrect.

    install
        Setting to True installs module

    source
        Points to module source file, used only when install is True

    remove
        Setting to True removes module

    .. versionadded:: 2016.3.0
    """
    ret = {"name": name, "result": True, "comment": "", "changes": {}}
    if opts.get("install", False) and opts.get("remove", False):
        ret["result"] = False
        ret["comment"] = "Cannot install and remove at the same time"
        return ret
    if opts.get("install", False):
        module_path = opts.get("source", name)
        ret = module_install(module_path)
        if not ret["result"]:
            return ret
    elif opts.get("remove", False):
        return module_remove(name)
    modules = __salt__["selinux.list_semod"]()
    if name not in modules:
        ret["comment"] = "Module {} is not available".format(name)
        ret["result"] = False
        return ret
    rmodule_state = _refine_module_state(module_state)
    if rmodule_state == "unknown":
        ret["comment"] = "{} is not a valid state for the {} module.".format(
            module_state, module
        )
        ret["result"] = False
        return ret
    if version != "any":
        installed_version = modules[name]["Version"]
        if not installed_version == version:
            ret["comment"] = (
                "Module version is {} and does not match "
                "the desired version of {} or you are "
                "using semodule >= 2.4".format(installed_version, version)
            )
            ret["result"] = False
            return ret
    current_module_state = _refine_module_state(modules[name]["Enabled"])
    if rmodule_state == current_module_state:
        ret["comment"] = "Module {} is in the desired state".format(name)
        return ret
    if __opts__["test"]:
        ret["result"] = None
        ret["comment"] = "Module {} is set to be toggled to {}".format(
            name, module_state
        )
        return ret

    if __salt__["selinux.setsemod"](name, rmodule_state):
        ret["comment"] = "Module {} has been set to {}".format(name, module_state)
        return ret
    ret["result"] = False
    ret["comment"] = "Failed to set the Module {} to {}".format(name, module_state)
    return ret


def module_install(name):
    """
    Installs custom SELinux module from given file

    name
        Path to file with module to install

    .. versionadded:: 2016.11.6
    """
    ret = {"name": name, "result": True, "comment": "", "changes": {}}
    if __salt__["selinux.install_semod"](name):
        ret["comment"] = "Module {} has been installed".format(name)
        return ret
    ret["result"] = False
    ret["comment"] = "Failed to install module {}".format(name)
    return ret


def module_remove(name):
    """
    Removes SELinux module

    name
        The name of the module to remove

    .. versionadded:: 2016.11.6
    """
    ret = {"name": name, "result": True, "comment": "", "changes": {}}
    modules = __salt__["selinux.list_semod"]()
    if name not in modules:
        ret["comment"] = "Module {} is not available".format(name)
        ret["result"] = False
        return ret
    if __salt__["selinux.remove_semod"](name):
        ret["comment"] = "Module {} has been removed".format(name)
        return ret
    ret["result"] = False
    ret["comment"] = "Failed to remove module {}".format(name)
    return ret


def fcontext_policy_present(
    name, sel_type, filetype="a", sel_user=None, sel_level=None
):
    """
    .. versionadded:: 2017.7.0

    Makes sure a SELinux policy for a given filespec (name), filetype
    and SELinux context type is present.

    name
        filespec of the file or directory. Regex syntax is allowed.

    sel_type
        SELinux context type. There are many.

    filetype
        The SELinux filetype specification. Use one of [a, f, d, c, b,
        s, l, p]. See also `man semanage-fcontext`. Defaults to 'a'
        (all files).

    sel_user
        The SELinux user.

    sel_level
        The SELinux MLS range.
    """
    ret = {"name": name, "result": False, "changes": {}, "comment": ""}
    new_state = {}
    old_state = {}
    filetype_str = __salt__["selinux.filetype_id_to_string"](filetype)
    current_state = __salt__["selinux.fcontext_get_policy"](
        name=name,
        filetype=filetype,
        sel_type=sel_type,
        sel_user=sel_user,
        sel_level=sel_level,
    )
    if not current_state:
        new_state = {name: {"filetype": filetype_str, "sel_type": sel_type}}
        if __opts__["test"]:
            ret.update({"result": None})
        else:
            add_ret = __salt__["selinux.fcontext_add_policy"](
                name=name,
                filetype=filetype,
                sel_type=sel_type,
                sel_user=sel_user,
                sel_level=sel_level,
            )
            if add_ret["retcode"] != 0:
                ret.update({"comment": "Error adding new rule: {}".format(add_ret)})
            else:
                ret.update({"result": True})
    else:
        if current_state["sel_type"] != sel_type:
            old_state.update({name: {"sel_type": current_state["sel_type"]}})
            new_state.update({name: {"sel_type": sel_type}})
        else:
            ret.update(
                {
                    "result": True,
                    "comment": 'SELinux policy for "{}" already present '.format(name)
                    + 'with specified filetype "{}" and sel_type "{}".'.format(
                        filetype_str, sel_type
                    ),
                }
            )
            return ret

        # Removal of current rule is not neccesary, since adding a new rule for the same
        # filespec and the same filetype automatically overwrites
        if __opts__["test"]:
            ret.update({"result": None})
        else:
            change_ret = __salt__["selinux.fcontext_add_policy"](
                name=name,
                filetype=filetype,
                sel_type=sel_type,
                sel_user=sel_user,
                sel_level=sel_level,
            )
            if change_ret["retcode"] != 0:
                ret.update({"comment": "Error adding new rule: {}".format(change_ret)})
            else:
                ret.update({"result": True})
    if ret["result"] and (new_state or old_state):
        ret["changes"].update({"old": old_state, "new": new_state})
    return ret


def fcontext_policy_absent(
    name, filetype="a", sel_type=None, sel_user=None, sel_level=None
):
    """
    .. versionadded:: 2017.7.0

    Makes sure an SELinux file context policy for a given filespec
    (name), filetype and SELinux context type is absent.

    name
        filespec of the file or directory. Regex syntax is allowed.

    filetype
        The SELinux filetype specification. Use one of [a, f, d, c, b,
        s, l, p]. See also `man semanage-fcontext`. Defaults to 'a'
        (all files).

    sel_type
        The SELinux context type. There are many.

    sel_user
        The SELinux user.

    sel_level
        The SELinux MLS range.
    """
    ret = {"name": name, "result": False, "changes": {}, "comment": ""}
    new_state = {}
    old_state = {}
    current_state = __salt__["selinux.fcontext_get_policy"](
        name=name,
        filetype=filetype,
        sel_type=sel_type,
        sel_user=sel_user,
        sel_level=sel_level,
    )
    if not current_state:
        ret.update(
            {
                "result": True,
                "comment": 'SELinux policy for "{}" already absent '.format(name)
                + 'with specified filetype "{}" and sel_type "{}".'.format(
                    filetype, sel_type
                ),
            }
        )
        return ret
    else:
        old_state.update({name: current_state})
    ret["changes"].update({"old": old_state, "new": new_state})
    if __opts__["test"]:
        ret.update({"result": None})
    else:
        remove_ret = __salt__["selinux.fcontext_delete_policy"](
            name=name,
            filetype=filetype,
            sel_type=sel_type or current_state["sel_type"],
            sel_user=sel_user,
            sel_level=sel_level,
        )
        if remove_ret["retcode"] != 0:
            ret.update({"comment": "Error removing policy: {}".format(remove_ret)})
        else:
            ret.update({"result": True})
    return ret


def fcontext_policy_applied(name, recursive=False):
    """
    .. versionadded:: 2017.7.0

    Checks and makes sure the SELinux policies for a given filespec are
    applied.
    """
    ret = {"name": name, "result": False, "changes": {}, "comment": ""}

    changes_text = __salt__["selinux.fcontext_policy_is_applied"](name, recursive)
    if changes_text == "":
        ret.update(
            {
                "result": True,
                "comment": (
                    'SElinux policies are already applied for filespec "{}"'.format(
                        name
                    )
                ),
            }
        )
        return ret
    if __opts__["test"]:
        ret.update({"result": None})
    else:
        apply_ret = __salt__["selinux.fcontext_apply_policy"](name, recursive)
        if apply_ret["retcode"] != 0:
            ret.update({"comment": apply_ret})
        else:
            ret.update({"result": True})
            ret.update({"changes": apply_ret.get("changes")})
    return ret


def port_policy_present(name, sel_type, protocol=None, port=None, sel_range=None):
    """
    .. versionadded:: 2019.2.0

    Makes sure an SELinux port policy for a given port, protocol and SELinux context type is present.

    name
        The protocol and port spec. Can be formatted as ``(tcp|udp)/(port|port-range)``.

    sel_type
        The SELinux Type.

    protocol
        The protocol for the port, ``tcp`` or ``udp``. Required if name is not formatted.

    port
        The port or port range. Required if name is not formatted.

    sel_range
        The SELinux MLS/MCS Security Range.
    """
    ret = {"name": name, "result": False, "changes": {}, "comment": ""}
    old_state = __salt__["selinux.port_get_policy"](
        name=name,
        sel_type=sel_type,
        protocol=protocol,
        port=port,
    )
    if old_state:
        ret.update(
            {
                "result": True,
                "comment": 'SELinux policy for "{}" already present '.format(name)
                + 'with specified sel_type "{}", protocol "{}" and port "{}".'.format(
                    sel_type, protocol, port
                ),
            }
        )
        return ret
    if __opts__["test"]:
        ret.update({"result": None})
    else:
        add_ret = __salt__["selinux.port_add_policy"](
            name=name,
            sel_type=sel_type,
            protocol=protocol,
            port=port,
            sel_range=sel_range,
        )
        if add_ret["retcode"] != 0:
            ret.update({"comment": "Error adding new policy: {}".format(add_ret)})
        else:
            ret.update({"result": True})
            new_state = __salt__["selinux.port_get_policy"](
                name=name,
                sel_type=sel_type,
                protocol=protocol,
                port=port,
            )
            ret["changes"].update({"old": old_state, "new": new_state})
    return ret


def port_policy_absent(name, sel_type=None, protocol=None, port=None):
    """
    .. versionadded:: 2019.2.0

    Makes sure an SELinux port policy for a given port, protocol and SELinux context type is absent.

    name
        The protocol and port spec. Can be formatted as ``(tcp|udp)/(port|port-range)``.

    sel_type
        The SELinux Type. Optional; can be used in determining if policy is present,
        ignored by ``semanage port --delete``.

    protocol
        The protocol for the port, ``tcp`` or ``udp``. Required if name is not formatted.

    port
        The port or port range. Required if name is not formatted.
    """
    ret = {"name": name, "result": False, "changes": {}, "comment": ""}
    old_state = __salt__["selinux.port_get_policy"](
        name=name,
        sel_type=sel_type,
        protocol=protocol,
        port=port,
    )
    if not old_state:
        ret.update(
            {
                "result": True,
                "comment": 'SELinux policy for "{}" already absent '.format(name)
                + 'with specified sel_type "{}", protocol "{}" and port "{}".'.format(
                    sel_type, protocol, port
                ),
            }
        )
        return ret
    if __opts__["test"]:
        ret.update({"result": None})
    else:
        delete_ret = __salt__["selinux.port_delete_policy"](
            name=name,
            protocol=protocol,
            port=port,
        )
        if delete_ret["retcode"] != 0:
            ret.update({"comment": "Error deleting policy: {}".format(delete_ret)})
        else:
            ret.update({"result": True})
            new_state = __salt__["selinux.port_get_policy"](
                name=name,
                sel_type=sel_type,
                protocol=protocol,
                port=port,
            )
            ret["changes"].update({"old": old_state, "new": new_state})
    return ret



"""
    tests.unit.payload_test
    ~~~~~~~~~~~~~~~~~~~~~~~
"""

import copy
import datetime
import logging

import salt.exceptions
import salt.payload
from salt.utils import immutabletypes
from salt.utils.odict import OrderedDict

log = logging.getLogger(__name__)


def assert_no_ordered_dict(data):
    if isinstance(data, OrderedDict):
        raise AssertionError("Found an ordered dictionary")
    if isinstance(data, dict):
        for value in data.values():
            assert_no_ordered_dict(value)
    elif isinstance(data, (list, tuple)):
        for chunk in data:
            assert_no_ordered_dict(chunk)


def test_list_nested_odicts():
    idata = {"pillar": [OrderedDict(environment="dev")]}
    odata = salt.payload.loads(salt.payload.dumps(idata.copy()))
    assert_no_ordered_dict(odata)
    assert idata == odata


def test_datetime_dump_load():
    """
    Check the custom datetime handler can understand itself
    """
    dtvalue = datetime.datetime(2001, 2, 3, 4, 5, 6, 7)
    idata = {dtvalue: dtvalue}
    sdata = salt.payload.dumps(idata.copy())
    odata = salt.payload.loads(sdata)
    assert (
        sdata
        == b"\x81\xc7\x18N20010203T04:05:06.000007\xc7\x18N20010203T04:05:06.000007"
    )
    assert idata == odata


def test_verylong_dump_load():
    """
    Test verylong encoder/decoder
    """
    idata = {"jid": 20180227140750302662}
    sdata = salt.payload.dumps(idata.copy())
    odata = salt.payload.loads(sdata)
    idata["jid"] = "{}".format(idata["jid"])
    assert idata == odata


def test_immutable_dict_dump_load():
    """
    Test immutable dict encoder/decoder
    """
    idata = {"dict": {"key": "value"}}
    sdata = salt.payload.dumps({"dict": immutabletypes.ImmutableDict(idata["dict"])})
    odata = salt.payload.loads(sdata)
    assert idata == odata


def test_immutable_list_dump_load():
    """
    Test immutable list encoder/decoder
    """
    idata = {"list": [1, 2, 3]}
    sdata = salt.payload.dumps({"list": immutabletypes.ImmutableList(idata["list"])})
    odata = salt.payload.loads(sdata)
    assert idata == odata


def test_immutable_set_dump_load():
    """
    Test immutable set encoder/decoder
    """
    idata = {"set": ["red", "green", "blue"]}
    sdata = salt.payload.dumps({"set": immutabletypes.ImmutableSet(idata["set"])})
    odata = salt.payload.loads(sdata)
    assert idata == odata


def test_odict_dump_load():
    """
    Test odict just works. It wasn't until msgpack 0.2.0
    """
    data = OrderedDict()
    data["a"] = "b"
    data["y"] = "z"
    data["j"] = "k"
    data["w"] = "x"
    sdata = salt.payload.dumps({"set": data})
    odata = salt.payload.loads(sdata)
    assert {"set": dict(data)}, odata


def test_mixed_dump_load():
    """
    Test we can handle all exceptions at once
    """
    dtvalue = datetime.datetime(2001, 2, 3, 4, 5, 6, 7)
    od = OrderedDict()
    od["a"] = "b"
    od["y"] = "z"
    od["j"] = "k"
    od["w"] = "x"
    idata = {
        dtvalue: dtvalue,  # datetime
        "jid": 20180227140750302662,  # long int
        "dict": immutabletypes.ImmutableDict({"key": "value"}),  # immutable dict
        "list": immutabletypes.ImmutableList([1, 2, 3]),  # immutable list
        "set": immutabletypes.ImmutableSet(("red", "green", "blue")),  # immutable set
        "odict": od,  # odict
    }
    edata = {
        dtvalue: dtvalue,  # datetime, == input
        "jid": "20180227140750302662",  # string repr of long int
        "dict": {"key": "value"},  # builtin dict
        "list": [1, 2, 3],  # builtin list
        "set": ["red", "green", "blue"],  # builtin set
        "odict": dict(od),  # builtin dict
    }
    sdata = salt.payload.dumps(idata)
    odata = salt.payload.loads(sdata)
    assert edata == odata


def test_recursive_dump_load():
    """
    Test recursive payloads are (mostly) serialized
    """
    data = {"name": "roscivs"}
    data["data"] = data  # Data all the things!
    sdata = salt.payload.dumps(data)
    odata = salt.payload.loads(sdata)
    assert "recursion" in odata["data"].lower()


def test_recursive_dump_load_with_identical_non_recursive_types():
    """
    If identical objects are nested anywhere, they should not be
    marked recursive unless they're one of the types we iterate
    over.
    """
    repeating = "repeating element"
    data = {
        "a": "a",  # Test CPython implementation detail. Short
        "b": "a",  # strings are interned.
        "c": 13,  # So are small numbers.
        "d": 13,
        "fnord": repeating,
        # Let's go for broke and make a crazy nested structure
        "repeating": [
            [[[[{"one": repeating, "two": repeating}], repeating, 13, "a"]]],
            repeating,
            repeating,
            repeating,
        ],
    }
    # We need a nested dictionary to trigger the exception
    data["repeating"][0][0][0].append(data)
    # If we don't deepcopy the data it gets mutated
    sdata = salt.payload.dumps(copy.deepcopy(data))
    odata = salt.payload.loads(sdata)
    # Delete the recursive piece - it's served its purpose, and our
    # other test tests that it's actually marked as recursive.
    del odata["repeating"][0][0][0][-1], data["repeating"][0][0][0][-1]
    assert odata == data


def test_raw_vs_encoding_none():
    """
    Test that we handle the new raw parameter in 5.0.2 correctly based on
    encoding. When encoding is None loads should return bytes
    """
    dtvalue = datetime.datetime(2001, 2, 3, 4, 5, 6, 7)
    idata = {dtvalue: "strval"}
    sdata = salt.payload.dumps(idata.copy())
    odata = salt.payload.loads(sdata, encoding=None)
    assert isinstance(odata[dtvalue], str)


def test_raw_vs_encoding_utf8():
    """
    Test that we handle the new raw parameter in 5.0.2 correctly based on
    encoding. When encoding is utf-8 loads should return unicode
    """
    dtvalue = datetime.datetime(2001, 2, 3, 4, 5, 6, 7)
    idata = {dtvalue: "strval"}
    sdata = salt.payload.dumps(idata.copy())
    odata = salt.payload.loads(sdata, encoding="utf-8")
    assert isinstance(odata[dtvalue], str)



import logging

from salt.transport import MessageClientPool
from tests.support.unit import TestCase

log = logging.getLogger(__name__)


class MessageClientPoolTest(TestCase):
    class MockClass:
        def __init__(self, *args, **kwargs):
            self.args = args
            self.kwargs = kwargs

    def test_init(self):
        opts = {"sock_pool_size": 10}
        args = (0,)
        kwargs = {"kwarg": 1}
        message_client_pool = MessageClientPool(
            self.MockClass, opts, args=args, kwargs=kwargs
        )
        self.assertEqual(
            opts["sock_pool_size"], len(message_client_pool.message_clients)
        )
        for message_client in message_client_pool.message_clients:
            self.assertEqual(message_client.args, args)
            self.assertEqual(message_client.kwargs, kwargs)

    def test_init_without_config(self):
        opts = {}
        args = (0,)
        kwargs = {"kwarg": 1}
        message_client_pool = MessageClientPool(
            self.MockClass, opts, args=args, kwargs=kwargs
        )
        # The size of pool is set as 1 by the MessageClientPool init method.
        self.assertEqual(1, len(message_client_pool.message_clients))
        for message_client in message_client_pool.message_clients:
            self.assertEqual(message_client.args, args)
            self.assertEqual(message_client.kwargs, kwargs)

    def test_init_less_than_one(self):
        opts = {"sock_pool_size": -1}
        args = (0,)
        kwargs = {"kwarg": 1}
        message_client_pool = MessageClientPool(
            self.MockClass, opts, args=args, kwargs=kwargs
        )
        # The size of pool is set as 1 by the MessageClientPool init method.
        self.assertEqual(1, len(message_client_pool.message_clients))
        for message_client in message_client_pool.message_clients:
            self.assertEqual(message_client.args, args)
            self.assertEqual(message_client.kwargs, kwargs)
